#! /usr/bin/python

#     file     : projects/PaminaIII/bin/RunIterativeReplanning.py
#     project  : Pamina III simulation suite
#     author   : Marcus Rickert
#     RCS      : $Id: GenerateTransimsRoutefile.py,v 1.1 2005/03/26 06:12:29 mr Exp $
#     purpose  : generate simple TRANSIMS routefile for the
#                PAMINA simulation suite
#
#     Copyright (C) 2005 :  Marcus Rickert

# TODO:
#


import random

from t_AppException import *
from t_OptionHandler import *

# options of the skript

OPTION_OD_FILE = "od-file=FILENAME"
OPTION_ROUTE_FILE = "route-file=FILENAME"

COL_FROM_NODE_ID = 0
COL_FIRST_LINK_ID = 1
COL_TO_NODE_ID = 2
COL_LAST_LINK_ID = 3 
COL_START_TIME = 4
COL_END_TIME = 5
COL_COUNT_PER_HOUR = 6
COL_COUNT = 7

# -----------------------------------------------------------------------------------

class OdPair:

    def __init__(self, p_Line):

        if p_Line[-1] == '\n':
            p_Line = p_Line[0:-1]

        l_Cols = string.split(p_Line, ',')
        if len(l_Cols) != COL_COUNT:
            raise t_AppException(p_Message = "invalid line '%s'" % l_Line)

        self.f_FromNodeId = l_Cols[COL_FROM_NODE_ID]
        self.f_FirstLinkId = l_Cols[COL_FIRST_LINK_ID]
        self.f_ToNodeId = l_Cols[COL_TO_NODE_ID]
        self.f_LastLinkId = l_Cols[COL_LAST_LINK_ID]
        self.f_StartTime = float(l_Cols[COL_START_TIME])
        self.f_EndTime = float(l_Cols[COL_END_TIME])
        self.f_CountPerHour = float(l_Cols[COL_COUNT_PER_HOUR])

        if self.f_StartTime >= self.f_EndTime:
            raise t_AppException(p_Message = "start time %f later than end time %f" % (self.f_StartTime, self.f_EndTime))

        self.f_RoutesPerSecond = int(self.f_CountPerHour / 3600.0)
        self.f_Prob = self.f_CountPerHour / 3600.0 - self.f_RoutesPerSecond
        self.f_TotalCount = self.f_CountPerHour * (self.f_EndTime - self.f_StartTime)
        self.f_Count = 0

        print "OD-Pair active from %f to %f with %d routes per hour (%d route per second + additional probability %f)." % \
              (self.f_StartTime, self.f_EndTime, self.f_CountPerHour, self.f_RoutesPerSecond, self.f_Prob)
        

    def WriteRoute(self, p_TimeStep, p_File):

        global g_PlanId
        global g_PlanAge

        p_File.write( " %d %d %d %d %s %d %s %d %s %d %s %d\n" %
                      (g_PlanId, g_PlanAge, g_PlanId, g_PlanId,
                       self.f_FromNodeId,
                       2, # number of links
                       self.f_FirstLinkId, p_TimeStep, # first link + estimate
                       self.f_LastLinkId, p_TimeStep,  # last link + estimate
                       self.f_ToNodeId, p_TimeStep + 1) )
        
        g_PlanId = g_PlanId + 1

    def CheckRoute(self, p_TimeStep, p_File):

        l_Time = p_TimeStep / 3600

        if l_Time < self.f_StartTime or l_Time >= self.f_EndTime:
            return 0

        l_Count = 0
        while l_Count < self.f_RoutesPerSecond:
            self.WriteRoute(p_TimeStep, p_File)
            l_Count = l_Count + 1

        l_Prob = (self.f_TotalCount - (self.f_Count + (self.f_EndTime - l_Time) * 3600 * self.f_RoutesPerSecond)) / \
                 ( (self.f_EndTime - l_Time) * 3600 )

        if random.random() < l_Prob:
            self.WriteRoute(p_TimeStep, p_File)
            l_Count = l_Count + 1

        self.f_Count = self.f_Count + l_Count

        return l_Count
        
                    

# -----------------------------------------------------------------------------------

def ReadOdFile():

    global g_OdPairs
    global g_MinTime
    global g_MaxTime

    l_File = open(g_OdFile, "r")

    print "Reading origin destination pairs from '%s'..." % g_OdFile

    g_OdPairs = [ ]
    g_MinTime = 24.0
    g_MaxTime = 0.0
    l_Line = l_File.readline()
    while (l_Line):

        l_OdPair = OdPair(l_Line)
        g_OdPairs.append(l_OdPair)

        if l_OdPair.f_StartTime < g_MinTime:
            g_MinTime = l_OdPair.f_StartTime

        if l_OdPair.f_EndTime > g_MaxTime:
            g_MaxTime = l_OdPair.f_EndTime

        l_Line = l_File.readline()

    l_File.close()

    print "Origin destination pairs read: %d" % len(g_OdPairs)
    print "Minimum time: %f" % g_MinTime
    print "Maximum time: %f" % g_MaxTime


        
# -----------------------------------------------------------------------------------

def WriteTransimsRoutes():

    global g_RouteFile
    global g_OdPairs
    global g_MinTime
    global g_MaxTime
    global g_PlanId

    print "Writing TRANSIMS routes to '%s'..." % g_RouteFile

    l_Routes = 0
    g_PlanId = 0

    l_File = open(g_RouteFile, "w")

    l_File.write("#   PLAN NO., MODE, VEHICLE ID, DRIVER PLAN NO., START NODE, NO. OF STEPS, LINK1, START TIME1, LINK2, START TIME2, ....., LAST LINK, LAST START TIME, END NODE, ARRIVAL TIME\n")
    l_TimeStep = 3600 * g_MinTime
    l_LastTimeStep = 3600 * g_MaxTime

    while l_TimeStep < l_LastTimeStep:

        for l_OdPair in g_OdPairs:

            l_Routes = l_Routes + l_OdPair.CheckRoute(l_TimeStep, l_File)

        l_TimeStep = l_TimeStep + 1

    l_File.close()

    print "Routes written to file: %d" % l_Routes


    
# -----------------------------------------------------------------------------------

def Main():

    ReadOdFile()
    WriteTransimsRoutes()


    
# -----------------------------------------------------------------------------------

if __name__ == '__main__':
    try:

        global g_PlanAge

        g_PlanAge = 0

        v_Options = [ (OPTION_OD_FILE, "input file with origin destination pairs"),
                      (OPTION_ROUTE_FILE, "TRANSIMS output file")]

        v_OptionHandler = t_OptionHandler(v_Options)
        
        v_OptionHandler.parse()

        print "Evaluating command line parameters..."

        g_OdFile = v_OptionHandler.getParameter(OPTION_OD_FILE)
        g_RouteFile = v_OptionHandler.getParameter(OPTION_ROUTE_FILE)
                
        Main()
        

    except IOError:
        printExceptionMessages()
            
    except t_EnvException:
        printExceptionMessages()
        v_OptionHandler.listOptions();

    except t_AppException, Exception:
        printExceptionMessages()
