/*      file     : TrafficData/TMultilaneStat.h
        project  : PAMINA
        author   : Marcus Rickert
        RCS      : $Id: TMultilaneStat.h,v 1.7 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TMultilaneStat_H
#define TMultilaneStat_H

/* ***INCLUDES************************************************************************ */

#include <stdio.h>

#include <TStatObject.h>
#include <CAConfig.h>

// ***FORWARDS************************************************************************

class TMultilaneGrid;

/* ***CLASSES************************************************************************* */

class TMultilaneStat : public TStatObject

{
 public:

    TMultilaneStat(TParallelID theParallelID = INVALID);
    virtual ~TMultilaneStat(void);
    
    bool
    Collect(const TMultilaneGrid *Grid);

    inline int
    GetMaxNrOfLanes(void) const;

    inline long
    GetNrOfActiveGrids(void) const;

    inline long
    GetNrOfGrids(void) const;

    inline long
    GetNrOfSites(int CLane = ALL_LANES) const;

    inline long
    GetNrOfActiveSites(void) const;

    inline long 
    GetNrOfSlowVehicles(int CLane = ALL_LANES) const;

    inline long 
    GetNrOfVehicles(int CLane = ALL_LANES, int CV = ALL_V) const;

    inline double
    GetSlowRatio(int CLane = ALL_LANES) const;

    inline long
    GetSumV(int CLane = ALL_LANES) const;
    
    inline double
    GetDensity(int CLane = ALL_LANES) const;

    inline double
    GetFlow(int CLane = ALL_LANES) const;

    double
    GetVDelta(int CLane = ALL_LANES) const;

    inline double
    GetVMean(int CLane = ALL_LANES) const;

    void
    PrintLineStat(TTimeStep RefTimeStep, FILE *OutputFile);

    void
    PrintLineStatDescription(FILE *OutputFile);

    virtual void
    PrintScreenStat(FILE *OutputFile);

    static TClassInfo *ClassInfo;

 private:


    virtual bool
    Accumulate(const TStatObject *StatObject);

    virtual bool
    Decode(TBaseStream *aStream, TTransferInfo *TransferInfo);

    virtual bool
    Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const;

    virtual bool
    Evaluate(void);

    virtual TString
    GetClassName(void) const;

    virtual void
    Reset(void);

    int  MaxNrOfLanes;
    long NrOfActiveSites;
    long NrOfGrids, NrOfActiveGrids;

    //  <<< DO NOT SEPERATE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#   define LONGINT_PER_LANE (CA_DEFAULT_V_MAX + 6)
    long NrOfSites[MAX_LANES+1];                       // 1
    long NrOfSlowVehicles[MAX_LANES+1];                // 1
    long V[MAX_LANES+1][CA_DEFAULT_V_MAX + 2];         // CA_DEFAULT_VA_MAX + 2
    long SumV[MAX_LANES + 1];                          // 1
    long SumV2[MAX_LANES + 1];                         // 1
    //  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
};

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TMultilaneStat.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
