/*     file     : TrafficData/TRouteplan.h
       project  : PAMINA III
       author   : Marcus Rickert
       RCS      : $Id: TRouteplan.h,v 1.9 2005/03/26 06:58:51 mr Exp $

       Copyright (C) 2005 Marcus Rickert

       http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TROUTEPLAN_H
#define TROUTEPLAN_H

// ***INCLUDES************************************************************************

#include <stdio.h>

#include <TArray.h>
#include <TDLList.h>
#include <ParToolsTypes.h>

#include <TParallelListObject.h>

// ***FORWARDS************************************************************************

class TBaseStream;
class TClassInfo;
class TTransferInfo;

// ***DEFINES*************************************************************************

#define MAX_ROUTE_LENGTH                2048

#define NR_OF_ROUTE_PLAN_TOKENS         1
#define TOKEN_ROUTE                     0

#define RP_FLAG_SOURCE_IS_ACCESSORY 1
#define RP_FLAG_DEST_IS_ACCESSORY   (RP_FLAG_SOURCE_IS_ACCESSORY << 1)
#define RP_FLAG_SHORTEN_ON_ENCODE   (RP_FLAG_SOURCE_IS_ACCESSORY << 2)

// ***CLASSES************************************************************************* 

extern "C" 
struct TRouteStep

{
    TParallelID ID;
    long        Estimate;
};

class TRouteplan : public TParallelListObject

{
 public:
    
    TRouteplan(TParallelID theParalleID);
    TRouteplan(TBaseStream *Stream, TTransferInfo *TransferInfo);
    TRouteplan(long theDepartureTimeStep, short int theFlags,
               TParallelID theSourceID, TBaseStream *Stream,
               TTransferInfo *TransferInfo);
    TRouteplan(long theDepartureTimeStep, short int theFlags,
               TParallelID theSourceID, 
               TParallelID theDestID, TParallelID theRouteID);
    virtual ~TRouteplan(void);

    bool
    AddStep(TParallelID ID, long Estimate);

    bool
    ContinuesWith(TRouteStep *RouteSteps, int NrOfRouteSteps) const;

    virtual TBaseObject *
    Copy(void) const;

    inline bool
    DestinationIsAccessory(void) const;

    virtual bool
    Decode(TBaseStream *aStream, TTransferInfo *);

    bool
    Detour(long CurrentTimeStep, long LastValidStep, TRouteStep *DetourSteps, 
           long NrOfDetourSteps, long DetourArrivalTime);

    virtual bool
    Encode(TBaseStream *aStream, TTransferInfo *) const;

    inline long
    GetActualTravelTime(void) const;

    inline long
    GetArrivalTimeStep(void) const;

    virtual TString
    GetClassName(void) const;

    TParallelID
    GetCurrentEdgeID(void);
    
    inline TParallelID
    GetCurrentID(void) const;

    inline long
    GetCurrentStep(void) const;

    inline long
    GetDeletionTimeStep(void) const;

    inline long
    GetEstimatedDeletionTimeStep(void) const;

    inline float
    GetEstimatedRelativeDelay(void) const;

    inline long
    GetDepartureTimeStep(void) const;

    inline TParallelID
    GetDestID(void) const;

    inline long
    GetEstimatedTravelTime(void) const;

    inline long
    GetInsertionTimeStep(void) const;

    virtual TString
    GetInstanceName(void) const;

    inline long
    GetNrOfReroutes(void) const;

    inline long
    GetNrOfSteps(void) const;

    inline long
    GetNrOfWholeSteps(void) const;

    inline long
    GetPlanLength(void) const;

    inline long
    GetRemainingPlanLength(void) const;

    inline TParallelID
    GetRouteID(void) const;

    inline TParallelID
    GetSourceID(void) const;

    inline TRouteStep &
    GetStep(long Step);

    inline bool
    IsLastAccessory(TParallelID AccessoryID) const;

    inline bool
    IsDestinationNode(TParallelID NodeID) const;

    bool
    Reroute(long CurrentTimeStep, TRouteStep *PlanSteps, int NrOfRouteSteps, 
            long RerouteArrivalTime);

    inline void
    SetDeletionTimeStep(long theDeletionTimeStep);

    inline void
    SetEstimatedRelativeDelay(float theEstimate);

    inline void
    SetInsertionTimeStep(long theInsertionTimeStep);

    bool
    Skip(TBaseStream *Stream);

    inline bool
    SourceIsAccessory(void) const;
    
    void
    Step(void);

    bool
    TerminatesWith(TRouteStep *RouteSteps, int NrOfRouteSteps) const;

  /*
    bool
    Write(TBaseStream *Stream);
    */

    bool
    WriteStat(TBaseStream *Stream);

    static TClassInfo *ClassInfo;

 private:

    TArray<TRouteStep>  Steps;

    //  >>> DO NOT SEPERATE AND DO NOT CHANGE ORDER <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#define TROUTEPLAN_VAR_LONG 7  
#define TROUTEPLAN_CONST_LONG 2 
#define TROUTEPLAN_CONST_LONG_ID 3
    // immutable members
    long          DepartureTimeStep;
    long          Flags;
    TParallelID   SourceID;
    TParallelID   DestID;
    TParallelID   RouteID;
    // mutable members
    long          CurrentStep;
    long          InsertionTimeStep;
    long          DeletionTimeStep;
    long          EstimatedTravelTime;
    long          LatestRerouteTimeStep;
    long          NrOfReroutes;
    long          OriginalEstimatedTravelTime;
#define TROUTEPLAN_VAR_FLOAT 1
    float         EstimatedRelativeDelay;
    //  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

    bool
    _Decode(TBaseStream *aStream, TTransferInfo *TransferInfo);

    bool
    DecodeSteps(TBaseStream *aStream);

    bool
    EncodeSteps(TBaseStream *aStream, int StepOffset) const;

    void
    _Init(void);
};

// ***TYPES*************************************************************************** 

typedef TDLList< TRouteplan > TRouteplanList;
typedef TDLListEnum< TRouteplan > TRouteplanEnum;

// ***EXTERNALS*********************************************************************** 

extern char *RouteplanTokens[];

// ***INLINE METHODS****************************************************************** 

#ifndef NO_INLINE
#  include "TRouteplan.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
