/*      file     : TrafficData/TRouteplan.i.C
        project  : PAMINA III
        authors  : Marcus Rickert
        RCS      : $Id: TRouteplan.i.C,v 1.4 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***METHODS************************************************************************* 

inline bool
TRouteplan::DestinationIsAccessory(void) const

{
    return (Flags & RP_FLAG_DEST_IS_ACCESSORY);
}

// ===================================================================================

inline long
TRouteplan::GetArrivalTimeStep(void) const

{
    return DepartureTimeStep + EstimatedTravelTime;
}

// ===================================================================================

inline long
TRouteplan::GetActualTravelTime(void) const

{
    return GetDeletionTimeStep() - GetInsertionTimeStep();
}


// ===================================================================================

inline long
TRouteplan::GetCurrentStep(void) const

{
    return CurrentStep;
}

// ===================================================================================

inline float
TRouteplan::GetEstimatedRelativeDelay(void) const

{
    return EstimatedRelativeDelay;
}

// ===================================================================================

inline TParallelID
TRouteplan::GetCurrentID(void) const

{
    if (CurrentStep < GetPlanLength())
        return Steps[CurrentStep].ID;
    else
        return INVALID_ID;
}

// ===================================================================================

inline long
TRouteplan::GetDeletionTimeStep(void) const

{
    return DeletionTimeStep;
}


// ===================================================================================

inline long
TRouteplan::GetDepartureTimeStep(void) const

{
    return DepartureTimeStep;
}


// ===================================================================================

inline TParallelID
TRouteplan::GetDestID(void) const

{
    return DestID;
}

// ===================================================================================

inline long
TRouteplan::GetEstimatedDeletionTimeStep(void) const

{
    return InsertionTimeStep + EstimatedTravelTime;
}

// ===================================================================================

inline long
TRouteplan::GetEstimatedTravelTime(void) const

{
    return EstimatedTravelTime;
}


// ===================================================================================

inline long
TRouteplan::GetNrOfReroutes(void) const

{
    return NrOfReroutes;
}

// ===================================================================================

inline long
TRouteplan::GetNrOfWholeSteps(void) const

{
    if (DestinationIsAccessory())
        return Steps.GetNrOfItems() - 1;
    else
        return Steps.GetNrOfItems();
}

// ===================================================================================

inline long
TRouteplan::GetNrOfSteps(void) const

{
    return Steps.GetNrOfItems();
}

// ===================================================================================

inline TParallelID
TRouteplan::GetRouteID(void) const

{
    return RouteID;
}

// ===================================================================================

inline TParallelID
TRouteplan::GetSourceID(void) const

{
    return SourceID;
}

// ===================================================================================

inline TRouteStep &
TRouteplan::GetStep(long Step)

{
    return Steps[Step];
}

// ===================================================================================

inline long
TRouteplan::GetInsertionTimeStep(void) const

{
    return InsertionTimeStep;
}


// ===================================================================================

inline long
TRouteplan::GetPlanLength(void) const

{
    return Steps.GetNrOfItems();
}

// ===================================================================================

inline long
TRouteplan::GetRemainingPlanLength(void) const

{
    return Steps.GetNrOfItems() - CurrentStep;
}

// ===================================================================================

inline bool
TRouteplan::IsLastAccessory(TParallelID AccessoryID) const

{
    return (CurrentStep == Steps.GetNrOfItems() AND
            AccessoryID == DestID AND
            DestinationIsAccessory());
}

// ===================================================================================

inline bool
TRouteplan::IsDestinationNode(TParallelID NodeID) const

{
    return (CurrentStep == Steps.GetNrOfItems() AND
            NodeID == DestID AND
            !DestinationIsAccessory());
}

// =================================================================================== 

inline void
TRouteplan::SetDeletionTimeStep(long theDeletionTimeStep) 

{
    DeletionTimeStep = theDeletionTimeStep;
}

// =================================================================================== 

inline void
TRouteplan::SetEstimatedRelativeDelay(float theEstimate)

{
    EstimatedRelativeDelay = theEstimate;
}

// =================================================================================== 

inline void
TRouteplan::SetInsertionTimeStep(long theInsertionTimeStep) 

{
    InsertionTimeStep = theInsertionTimeStep;
    if (Steps.GetNrOfItems())
        EstimatedTravelTime = Steps[Steps.GetNrOfItems() - 1].Estimate - DepartureTimeStep;
}

// ===================================================================================

inline bool
TRouteplan::SourceIsAccessory(void) const

{
    return (Flags & RP_FLAG_SOURCE_IS_ACCESSORY);
}

// =================================================================================== 

