/*      file     : TrafficData/TShortestPath.h
        project  : PAMINA III
        authors  : Marcus Rickert
        RCS      : $Id: TShortestPath.h,v 1.3 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TSHORTESTPATH_H
#define TSHORTESTPATH_H

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>
#include <TObjectHandler.h>
#include <THeap.h>

#include "TRouteplan.h"
#include "TAlgoInfo.h"
#include "TAlgoEdge.h"

// ***FORWARDS************************************************************************

class TAlgoEdge;
class TAlgoInfo;
class TClassInfo;
class TGraphPrimitive;
class TGridStatLookup;
class TStream;
class TTimeStep;

#include <TNode.forward.h>
#include <TEdge.forward.h>

// ***DEFINES*************************************************************************
// ***CONSTANTS***********************************************************************

enum TShortestPathError

{
    SP_OK,
    SP_CannotAllocate,
    SP_AlgoInfoNotAvailable,
    SP_TravelTimeNotAvailable,
    SP_NoPath,
    SP_PathTooLong
};

// ***CLASSES************************************************************************* 

struct TNetAccessInfo

{
    TNode     *Node;
    TEdge     *Edge;
    long       Time;
    long       AlgoInfoTime;
    TAlgoInfo *AlgoInfo;
    double     FractionOnEdge;
};

// ===================================================================================

class TShortestPath : public TBaseObject, public TObjectHandler

{
 public:

    TShortestPath(long theStartTimeStep, TClassInfo *theLookupClassInfo);
    virtual ~TShortestPath(void);

    bool
    BuildTree(TNetAccessInfo *AccessInfos, int NrOfAccessInfos);

    bool
    EstimateRemainingTravelTime(long CurrentTimeStep, TParallelID EdgeID,
                                double RelativePosition, TRouteplan *Route,
                                int &LastValidStep, long &EstimatedArrivalTimeStep);

    TShortestPathError
    GetError(void) const;

    bool
    GetPath(TNetAccessInfo *AccessInfos, int NrOfAccessInfos,
            TRouteStep *RouteSteps, int MaxRouteSteps, int &NrOfRouteSteps,
            TNetAccessInfo *&DestAccessInfo);

    bool
    IsActive(void) const;

 private:

    bool
    Allocate(void);

    void
    Deallocate(void);

    bool
    DetermineLocalNetwork(void);

    long
    GetAlgoEdgeIndex(TEdge *Edge, bool IssueWarning = FALSE);

    long
    GetAlgoInfoIndex(TParallelID SearchID, bool IssueWarning);

    TAlgoInfo *
    GetAlgoInfo(TNode *Node, bool IssueWarning = FALSE);

    virtual TString
    GetClassName(void) const;

    double
    GetEdgeTravelTime(TEdge *Edge, long TimeStep);

    virtual bool
    HandleAction(TParallelObject *theClassInfo, TObjectAction theAction);

    virtual bool
    HandleAnonymousAction(TClassInfo *theClassInfo, TObjectAction theAction);

    bool
    ProcessNetAccessInfos(TNetAccessInfo *Infos, int NrOfInfos, bool SetTime);

    void
    ResetAlgoInfos(void);

    TNetAccessInfo      PreviousAccessInfo;
    long                SourceTimeSlack;
    long                SourceTimeOffset;
    double              ManhattenLimit;
    TShortestPathError  LatestError;
    TClassInfo         *LookupClassInfo;
    long                StartTimeStep;
    TNodeList           Nodes;
    TEdgeList           Edges;
    TAlgoInfo          *AlgoInfos;
    TAlgoEdge          *AlgoEdges;
    long               *Incidences;
    TGridStatLookup    *GridStats;

    TAlgoInfo           SearchNode;
    TAlgoEdge           SearchEdge;
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
