/*      file     : TrafficData/TTrafficDataManager.h
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TTrafficDataManager.h,v 1.10 2005/05/01 14:57:58 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TTRAFFICDATAMANAGER_H
#define TTRAFFICDATAMANAGER_H

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>
#include <TObjectHandler.h>
#include <TStatHandler.h>

// ***FORWARDS************************************************************************

class TAccessoryHandler;
class TAdjacenceInfo;
class TAsciiDataTypeReader;
class TDatabaseNode;
class TDatabaseEdge;
class TEdgeInfo;
class TIntersectionControl;
class TLocationInfo;
class TObjectTimeStat;
class TPreviousNode;

#include <TNode.forward.h>
#include <TEdge.forward.h>

// ***DEFINES*************************************************************************

#define TRAFFIC_NODES       0x00000001
#define TRAFFIC_EDGES       0x00000002
#define TRAFFIC_ACCESSORIES 0x00000004
#define TRAFFIC_PHASES      0x00000008
#define TRAFFIC_EDGE_TIMES  0x00000010
#define TRAFFIC_NODE_TIMES  0x00000020
 
// ***CLASSES************************************************************************* 

class TTrafficDataManager : public TBaseObject, public TObjectHandler, public TStatHandler

{
public:

    TTrafficDataManager(void);
    virtual ~TTrafficDataManager(void);

    bool
    Activate(void);

    bool
    CheckDataFiles(long OpenMask);

    bool
    CreateSimulationObjects(TNodeList *NodeList, TEdgeList *EdgeList,
                            TInfoMask NodeLocalMask = 0, TInfoMask EdgeLocalMask = 0);

    TInfoMask
    GetDistributionEdgeLoadableMask(void) const;

    TInfoMask
    GetDistributionNodeLoadableMask(void) const;

    bool
    Run(void);

private:

    bool
    ActivateStatistics(void);

    virtual bool
    Collect(TSamplingDescription *Description);

    bool
    CollectObjectTimers(const TTimeStep &TimeStep, TStatObjectList *StatList,
                        TClassInfo *TimerClass, TClassInfo *TimeStatClass);

    bool
    LoadIntersectionControl(TIntersectionControl *IntersectionControl);

    void
    Dispose(void);

    virtual TString
    GetClassName(void) const;

    bool
    GetEdgeIndexArray(TParallelID *&IndexArray, long &Count);

    bool
    GetNodeIndexArray(TParallelID *&IndexArray, long &Count);
    
    virtual bool
    HandleCreateRequest(TParallelObject *&Object, const TClassInfo *ClassInfo, 
                        TParallelID ParallelID);

    virtual bool
    HandleAggregationResult(TAggregationInfo *AggregationInfo, TTimeStep &TimeStep);

    bool
    HandleTimeResult(TAggregationInfo *AggregationInfo, TTimeStep &TimeStep, TStream *Output);

    virtual bool
    HandleLoadRequest(TParallelObject *Object, bool &Loaded);

    void
    _Init(void);

    bool
    LoadAdjacenceInfo(TAdjacenceInfo *AdjacenceInfo);

    bool
    LoadAccessoryHandler(TAccessoryHandler *AccessoryHandler);

    bool
    LoadEdgeInfo(TEdgeInfo *EdgeInfo);

    bool
    LoadEdgeTimeStat(TObjectTimeStat *TimeStat);

    bool
    LoadIncidenceInfo(TIncidenceInfo *IncidenceInfo);

    bool
    LoadLocationInfo(TLocationInfo *LocationInfo);

    bool
    LoadNodeTimeStat(TObjectTimeStat *TimeStat);

    bool
    Open(void);

    long                  FileOpenMask;

    TString               BaseFilename;

    TAsciiDataTypeReader *NodeReader;
    TInfoMask             DistributionNodeLoadableMask;
    TInfoMask             NodeLoadableMask;
    TInfoMask             NodeRequiredMask;
    TAsciiDataTypeReader *EdgeReader;
    TAsciiDataTypeReader *IncomingEdgeReader;
    TAsciiDataTypeReader *OutgoingEdgeReader;
    TInfoMask             DistributionEdgeLoadableMask;
    TInfoMask             EdgeLoadableMask;
    TInfoMask             EdgeRequiredMask;
    TAsciiDataTypeReader *AccessoryReader;
    TAsciiDataTypeReader *PhaseReader;
    double                Granularity;

    TStream               *EdgeTimeStatOutput;
    TAsciiDataTypeReader  *EdgeTimeStatReader;
    int                    EdgeTimeStatInterval;
    TSamplingDescription  *EdgeTimeSamplingDescription;
    TAggregationInfo      *EdgeTimeAggregationInfo;
    TStream               *NodeTimeStatOutput;
    TAsciiDataTypeReader  *NodeTimeStatReader;
    int                    NodeTimeStatInterval;
    TSamplingDescription  *NodeTimeSamplingDescription;
    TAggregationInfo      *NodeTimeAggregationInfo;
};

// ***EXTERNALS*********************************************************************** 

extern TTrafficDataManager *TrafficDataManager;

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
