/*      file     : XTrafficData/TMapViewer.h
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TMapViewer.h,v 1.4 2005/03/25 12:29:05 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TMAPVIEWER_H
#define TMAPVIEWER_H

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>

// ***FORWARDS************************************************************************

class TRectangle;
class TTimeStep;

#include "TNode.forward.h"
#include "TEdge.forward.h"
#include "TMapView.forward.h"

// ***CLASSES************************************************************************* 

class TMapViewer : public TBaseObject, public TObjectHandler

{
public:

    TMapViewer(TRectangle *theFrame, TNodeList *theNodes, TEdgeList *theEdges);
    virtual ~TMapViewer(void);
    
    bool
    ExecuteSubTimeStep(const TTimeStep &TimeStep);

    void
    CloseView(TMapView *View);

    TMapViewList *
    GetMapViewList(void);

    void
    MarkViews(void);

    bool
    OpenView(TRectangle Frame);

    bool
    Run(void);

    bool
    UpdateNode(TNode *Node);

private:

    bool
    CheckOpen(void);

    void
    CloseViews(TMapViewList *List);

    bool
    ColorEdge(TEdge *Edge);
    
    bool
    ColorEdges(void);
    
    bool
    ColorCPN(void);

    virtual TString
    GetClassName(void) const;

    virtual TString
    GetInstanceName(void) const;

    virtual bool
    HandleAction(TParallelObject *Object, TObjectAction theAction);

    virtual bool
    HandleAnonymousAction(TClassInfo *ClassInfo, TObjectAction theAction);

    TString        Name;
    int            Width, Height;
    bool           TopologyChanged;

    TRectangle    *Frame;
    TNodeList     *Nodes;
    TEdgeList     *Edges;
    TMapViewList   Views;
    TMapViewList   ViewsToBeClosed;
    int            DumpInterval;
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
