/*      file     : common/LexYacc.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: LexYacc.C,v 1.4 2005/03/29 17:12:48 mr Exp $

        Copyright (C) 2005 Marcus Rickert
        
        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <cassert>

#include "LexYacc.h"

#include "TDebug.h"
#include "TConfiguration.h"
#include "TEnvironment.h"

// ***LOCAL VARIABLES***************************************************************** 

static TString CGroupname;
static TString CName;
static TString CString;

#ifdef HAS_YYLINENO
extern int yylineno;
#endif

// ***FUNCTIONS*********************************************************************** 

void
LexDebug(const char *aString ...)

{
    debug(PrintDebug(DM_LEX, aString));
}

// =================================================================================== 

void
LexSetConfig(void)

{
    assert(CGroupname.GetPtr() != 0);
    assert(CName.GetPtr() != 0);
    assert(CString.GetPtr() != 0);
    
    Configuration->Set(TConfigEntry(CGroupname, CName, CString()));
}

// =================================================================================== 

void
LexSetConfigString(const char *aString)

{
    assert(CGroupname.GetPtr() != 0);
    assert(CName.GetPtr() != 0);
    assert(CString.GetPtr() != 0);
    
    Configuration->Set(TConfigEntry(CGroupname, CName, aString));
}

// =================================================================================== 

void
LexSetGroupname(void)

{
    CGroupname = CName;
}

// =================================================================================== 

void
LexSetName(const char *aName)

{
    assert(aName != 0);

    CName = aName;
}

// =================================================================================== 

void
LexSetString(const char *aString)

{
    assert(((void *) aString) != 0);

    CString = SubString(aString, 1, -1);
}

// =================================================================================== 

void
LexSetStringValue(const char *aString)

{
    assert(((void *) aString) != 0);

    CString = aString;
}

// =================================================================================== 

void
YaccDebug(const char *aString ...)

{
    debug(PrintDebug(DM_YACC, aString));
}

// =================================================================================== 
#ifdef NO_MBS_INVALID
int
mbsinvalid(const char *)
{
    return FALSE;
}
#endif
// =================================================================================== 

int 
yywrap(void)

{
    return 1;
}

// =================================================================================== 

void
yyerror(char *ErrorString)

{
    int line_number = -1;

#ifdef HAS_YYLINENO    
    line_number = yylineno;
#endif

    //! \todo better report of line number (yylineno seems to be wrong)
    Configuration->ParseError(ErrorString, line_number);
}

// =================================================================================== 

