/*      file     : common/TConfig.h
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TConfig.h,v 1.5 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TCONFIG_H
#define TCONFIG_H

// ***INCLUDES************************************************************************

#include <TBaseObject.h>
#include <TConfigHandler.h>

// ***FORWARDS************************************************************************

class TParser;

// ***CLASSES*************************************************************************

/** Class handling configuration files. 
  Outdated, replaced by TConfiguration.
  */

class TConfig : public TBaseObject, public TConfigHandler

{
public:

    TConfig(void);
    virtual ~TConfig(void);

    bool
        CommandLineConfigItem(const char *Option);
    bool
        Load(const char *ConfigName, bool MustExist = FALSE);
    void
        Register(TConfigHandler *Handler);
    void
        SignatureHeader(const char *header) const;
    void
        _SignatureLine(const char *line) const;
    void
        WriteCompleteSignature(FILE *file) const;

private:
    
    TConfigHandlerList Handlers;

    MUTABLE_DECL FILE  *SignatureFile;
    int                 TokenCount;
    char              **Tokens;
    TParser            *CurrentParser;

    void
        BuildTokens(void);
    bool
        CheckReadAccess(const char *Filename) const;
    void
        DisposeTokens(void);
    virtual TString
        GetClassName(void) const;
    virtual int
        GetConfigEntryCount(void) const;
    virtual TString
        GetInstanceName(void) const;
    virtual bool
        HandleConfigItem(int ID, const char *, const char *Value);
    bool
        HandleItem(TConfigHandler *Handler, int ID, const char *Value);
    bool
        Read(TParser *Parser);
    virtual void
        RegisterTokens(int &Index, char **Tokens);
    virtual void
        WriteSignature(void) const;
};
 
// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
