/*      file     : common/TConfiguration.i.C
        project  : General Toolbox
        authors  : Marcus Rickert
        RCS      : $Id: TConfiguration.i.C,v 1.7 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#include <cassert>

#include <stdio.h>
#include <stdlib.h>

// ***METHODS************************************************************************* 

inline TString
TConfigEntry::GetKey(void) const

{
    assert(Groupname() != 0);
    assert(Name() != 0);
    assert(Groupname.Length() <= 128);
    assert(Name.Length() <= 128);

    char Line[1024];

    sprintf(Line, "%s.%s", Groupname(), Name());
    return Line;
}

// =================================================================================== 

inline const char *
TConfigEntry::GetValue(void) const

{
    return Value.GetPtr();
}

// =================================================================================== 

inline void
TConfigEntry::SetDefault(const char *aDefault)

{
    Default = aDefault;
}

// =================================================================================== 

inline void
TConfigEntry::SetValue(const char *aValue)

{
    Value = aValue;
}

// =================================================================================== 

inline double
TConfiguration::GetDouble(const char *Groupname, const char *Name)

{
    double      Value;
    const char *String;

    String = GetString(Groupname, Name);
    if (sscanf(String, "%lf", &Value) != 1)
    {
        Warning("config entry [%s].%s has invalid value '%s'", 
                Groupname, Name, String);
        return (double) INVALID;
    }
    return Value;
}

// =================================================================================== 

inline long
TConfiguration::GetLong(const char *Groupname, const char *Name)

{
    long        Value;
    const char *String;

    String = GetString(Groupname, Name);
    if (sscanf(String, "%ld", &Value) != 1)
    {
        Warning("config entry [%s].%s has invalid value '%s'", 
                Groupname, Name, String);
        return (long) INVALID;
    }
    return Value;
}

// =================================================================================== 

inline bool
TConfiguration::Exists(const char *Groupname, const char *Name)

{
    PreviousEntry = TConfigEntry(Groupname, Name);

    return Table.Contains((char *) PreviousEntry.GetKey(), CONFIG_TABLE_STRCMP);
}

// =================================================================================== 
