/*      file     : common/TSignal.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TSignal.C,v 1.10 2005/03/25 11:05:00 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

/* ***INCLUDES************************************************************************ */

#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

#include <TDebug.h>
#include <TSignal.h>

/* ***LOCAL VARIABLES***************************************************************** */

TSignal *SignalObject = NULL;

/* ***FUNCTIONS*********************************************************************** */

void
PrintError(const char *String)

{
    if (SignalObject)
        SignalObject->HandleErrorString(String);
    else
        fprintf(stderr, String);
}

/* =================================================================================== */

bool
SetSignalObject(TSignal *anObject)

{
    SignalObject = anObject;
    return TRUE;
}

/* =================================================================================== */

void
MySignalHandler(int SignalNr, ...)

{
    if (SignalObject)
        SignalObject->HandleSignal(SignalNr);
}

/* ***METHODS************************************************************************* */

TSignal::TSignal(void)

{
    int i;

    for (i = 0; i < MAX_SIGNAL; i++)
    {
        OldHandlers[i] = 0;
        SignalActive[i] = FALSE;
    }
}

/* =================================================================================== */

TSignal::~TSignal(void)

{
    RestoreAllSignals();
}

/* =================================================================================== */

bool
TSignal::CatchSignal(int SignalNr)

{
#ifndef NO_SIGNAL
    struct sigaction SigVec;

#  ifdef __SUNPRO_CC
#    ifdef __SunOS_5_4
     // works for Sisyphos:
#      define SIG_HANDLER_CAST void (*)()
#    else
     // works for Duraflame
#      define SIG_HANDLER_CAST void (*)(int)
#    endif
#  endif

#  if defined (__CRAY_CC) || defined(__SGI_CC)
#    define SIG_HANDLER_CAST void (*)(int)
#  endif

#  ifndef SIG_HANDLER_CAST
#    define SIG_HANDLER_CAST void (*)(int)
#  endif

    SigVec.sa_handler =  (SIG_HANDLER_CAST) MySignalHandler;
    sigemptyset(&SigVec.sa_mask);
    sigaddset(&SigVec.sa_mask, SignalNr - 1);

    SigVec.sa_flags = 0;

    debug(Print(DM_SIGNAL, "CatchSignal"));
    RestoreSignal(SignalNr);
    if (sigaction(SignalNr, &SigVec, OldActions+SignalNr) != 0)
    {
        Error("CatchSignal: cannot trap signal %d", SignalNr);
        return FALSE;
    }
#else
    Warning("platform does not support signal handling -> cannot catch signal %d", SignalNr);
#endif
    SignalActive[SignalNr] = TRUE;
    return TRUE;

}

/* =================================================================================== */

void
TSignal::HandleErrorString(const char *String)

{
    fprintf(stderr, String);
}

/* =================================================================================== */

int
TSignal::HandleSignal(int SignalNr)

{
    switch(SignalNr)
    {
    case SIGSEGV:
    case SIGBUS:
        exit(2);
    default:
        ;
    }
    return 0; 
}

/* =================================================================================== */

void
TSignal::RestoreAllSignals(void)

{
    int i;

    for (i = 0; i < MAX_SIGNAL; i++)
        if (SignalActive[i])
            RestoreSignal(i);
}

/* =================================================================================== */

void
TSignal::RestoreSignal(int SignalNr)

{

#ifdef DSIGNAL
    int i;
    Trace("RestoreSignal");

    for (i = 0; i < MAX_SIGNAL; i++)
        if (SignalActive[i])
            Print("Signal %d OldHandler %d", i, int (OldHandlers[i]));
#endif
    
    if (SignalActive[SignalNr])
    {
        debug(Print(DM_SIGNAL, "Signal restored Value:%d", OldHandlers[SignalNr]));
#ifndef NO_SIGNAL
        if (sigaction(SignalNr, OldActions+SignalNr, NULL) != 0)
            Error("CatchSignal: cannot restore signal %d", SignalNr);
        
        /*
          signal(SignalNr, OldHandlers[SignalNr]);

          the signal command cannot be used at the moment since the object identifier
          produced by gcc on Faure cannot be found in the c-library.
          
        */
#else
        Warning("platform does not support signal handling -> cannot catch signal %d", SignalNr);
#endif
        SignalActive[SignalNr] = FALSE;
    }
}

/* =================================================================================== */

