/*      file     : common/TSynchronize.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TSynchronize.C,v 1.5 2005/03/25 11:05:00 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include "TSynchronize.h"

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#endif

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

TSynchronize::TSynchronize(void)

{
    Read = 0;
    Write = 0;
}

// =================================================================================== 

TSynchronize::~TSynchronize(void)

{
    Deactivate();
}

// =================================================================================== 

bool
TSynchronize::Activate(const char *thePipeName, bool IsMasterFlag)

{
    char ReadFilename[128];
    char WriteFilename[128];

    PipeName = thePipeName;
    IsMaster = IsMasterFlag;

    if (IsMaster)
    {
        sprintf(WriteFilename, "%s.master", PipeName());
        sprintf(ReadFilename, "%s.slave", PipeName());
    }
    else
    {
        sprintf(ReadFilename, "%s.master", PipeName());
        sprintf(WriteFilename, "%s.slave", PipeName());
    }
    if ((Read = open(ReadFilename, O_RDONLY | O_NONBLOCK)) == -1)
    {
        Error("Activate: cannot open pipe %s for reading", ReadFilename);
        Read = 0;
        return FALSE;
    }
    if ((Write = open(WriteFilename, O_WRONLY)) == -1)
    {
        Error("Activate: cannot open pipe %s for writing", WriteFilename);
        Write = 0;
        return FALSE;
    }
    return TRUE;
}

// =================================================================================== 

bool
TSynchronize::Deactivate(void)

{
    if (Write)
    {
        close(Write);
        Write = 0;
    }
    sleep(1);
    if (Read)
    {
        close(Read);
        Read = 0;
    }
    return TRUE;
}

// =================================================================================== 

bool
TSynchronize::Step(long Step)

{
    long NrOfBytes;

    if (Write == 0)
    {
        Error("Step: not active");
        return FALSE;
    }
    if ((NrOfBytes = write(Write, &Step, sizeof(long))) != sizeof(long))
    {
        Error("Step: cannot write to pipe (%ld bytes written)", NrOfBytes);
        return FALSE;
    }
    return TRUE;
}

// =================================================================================== 

bool
TSynchronize::Wait(long &MyStep, bool &ErrorFlag)

{
    long Step;
    long NrOfBytes;

    ErrorFlag = FALSE;
    if (Read == 0)
    {
        Error("Wait: not active");
        ErrorFlag = TRUE;
        return FALSE;
    }
    
    if ((NrOfBytes = read(Read, &Step, sizeof(long))) != sizeof(long))
    {
        if (errno == EAGAIN)
        {
            errno = 0;
            return TRUE;
        }
        else
        {
            Error("Wait: read returns errno=%d NrOfBytes=%ld", errno, NrOfBytes);
            ErrorFlag = TRUE;
            return FALSE;
        }
    }
    if (Step >= MyStep)
    {
        MyStep = Step;
        return FALSE;
    }
    return TRUE;
}

// =================================================================================== 
#undef ParentClass
