/*      file     : graphx/MyGrTest.C
        project  : graphical toolbox
        author   : Michael Olesen
        comments : simple classes for using X11
        RCS      : $Id: MyGrTest.C,v 1.2 2005/03/25 11:06:10 mr Exp $
*/

#include <Graphx.h>

INT main(INT argc,CHAR **argv) {
  CHAR title[80];
  INT m,n,i;
  REAL x,xOld,x_min,x_max,mu,mu_min,mu_max,xT,yT;
  GrWindow *theWindow;

  if (argc != 6) {
    fprintf(stderr,"Usage: logistic xMin xMax muMin muMax fontpattern.\n\a");
    exit(1);
  }
  
  sscanf(argv[1],"%lf",&x_min);
  sscanf(argv[2],"%lf",&x_max);
  sscanf(argv[3],"%lf",&mu_min);
  sscanf(argv[4],"%lf",&mu_max);

  sprintf(title,"The logistic map: x->%c*x*(1-x). %g < %c < %g and %g < x < %g.",
    181,min(mu_min,mu_max),181,max(mu_min,mu_max),min(x_min,x_max),max(x_min,x_max));
  theWindow = new GrWindow(title);

  if (theWindow == NULL) {
    fprintf(stderr,"Unable to create graphics window.\n");
    exit(1);
  }
  
  printf("Fonts available in window: %d/%d.\n",theWindow->TopFont(),
    theWindow->NoFonts());
  theWindow->LoadFont(argv[5]);
  printf("Fonts available in window: %d/%d.\n",theWindow->TopFont(),
    theWindow->NoFonts());
  theWindow->SetFont(0);
  theWindow->SetHorizontalTextAlignment(GrRightAlign);
  
  theWindow->X11SetSpeed(GrFast);
  theWindow->SetXaxis(min(mu_min,mu_max),max(mu_min,mu_max));
  theWindow->SetYaxis(min(x_min,x_max),max(x_min,x_max));
  
  theWindow->X11WaitUntilExposed();
  
  theWindow->ClearScreen(255);
  
  m = 0;
  while(1) {
    mu = mu_min+m/500.0*(mu_max-mu_min);
    if (rand()%10==0)
      *theWindow << GrChar(xT=mu_min+(rand()%1000)/1000.0*(mu_max-mu_min),
        yT=x_min+(rand()%1000)/1000.0*(x_max-x_min),'A'+rand()%(256-'A'),
        rand()%256);
      *theWindow << GrLine(xT-0.005,yT,xT+0.005,yT,0);
      *theWindow << GrLine(xT,yT-0.05,xT,yT+0.05,0);
    for (n = 0;n < 10;n++) {
      x = x_min+(rand()%1000)/1000.0*(x_max-x_min);
      for (i = 0;i < 1000;i++) x *= mu*(1-x);
      *theWindow << GrPoint(mu,x,0);
      for (i = 0;i < 100;i++) {
        xOld = x;
        x *= mu*(1-x);
        *theWindow << GrPoint(mu,x,0);
        if (fabs(x-xOld)<1e-4) break;
      }
    }
    m++;
    m %= 500;
  }
}
