/*      file     : graphx/TWindow.C
        project  : General Toolbox
        author   : Marcus Rickert and Michael Olesen
        RCS      : $Id: TWindow.C,v 1.11 2005/03/25 11:06:10 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

/* ***INCLUDES************************************************************************ */

#include <Basic.h>
#include <TLocation.h>
#include <TWindow.h>

/* ***DEFINES************************************************************************* */

#define ParentClass GrWindow

/* ***METHODS************************************************************************* */

#ifdef NO_INLINE
#  include "TWindow.i.C"
#endif

/* =================================================================================== */

TWindow::TWindow(const char *WindowName, int Width, int Height, int ReservedColors) :

ParentClass((char *) WindowName, Width, Height, ReservedColors)

{
    _Init();
}
/* =================================================================================== */
TWindow::~TWindow(void)
{}

/* =================================================================================== */

bool
TWindow::Display(void)

{
    if (Redisplay())
    {
        DumpBackground(1);
        X11Flush();
        RedisplayFlag = FALSE;
    }
    if (DumpActive AND DumpFlag)
    {
        DumpFlag = FALSE;
        if (!Dump())
        {
            Error("during dump");
            return FALSE;
        }
    }
    return TRUE;
}

/* =================================================================================== */

bool
TWindow::Draw(void)

{
    if (DumpActive AND DumpFlag)
        LabelDump(GetDumpLabel());
    RedrawFlag = FALSE;
    return TRUE;
}

// ===================================================================================

void
TWindow::DrawArrow(const TLocation &From, double FromGap, 
                   const TLocation &To, double ToGap,
                   double Width, bool DrawTip, int Color)

{
    XPoint Points[5];
    TLocation Vector (To - From);
    TLocation Normal (Vector.Normal());

    Normal *= (0.5 * Width / Vector.Length());
    double Length = Vector.Length();
    TLocation NewFrom (From + FromGap / Length * Vector);
    TLocation NewTo (To - ToGap /Length * Vector);
    TLocation P1 (NewFrom + Normal);
    TLocation P2 (NewFrom - Normal);
    Points[0].x = X(theWindow, P1.x());
    Points[0].y = Y(theWindow, P1.y());
    Points[1].x = X(theWindow, P2.x());
    Points[1].y = Y(theWindow, P2.y());
    if (DrawTip)
    {
        TLocation P3 (0.1 * NewFrom + 0.9 * NewTo - Normal);
        TLocation P4 (0.1 * NewFrom + 0.9 * NewTo + Normal);
        Points[2].x = X(theWindow, P3.x());
        Points[2].y = Y(theWindow, P3.y());
        Points[3].x = X(theWindow, NewTo.x());
        Points[3].y = Y(theWindow, NewTo.y());
        Points[4].x = X(theWindow, P4.x());
        Points[4].y = Y(theWindow, P4.y());
        (*this) << GrFillPolygon(Points, 5, Convex, CoordModeOrigin, Color);
        (*this) << GrFillPolygon(Points + 2, 3, Convex, CoordModeOrigin, 0);
    }
    else
    {
        TLocation P3 (NewTo - Normal);
        TLocation P4 (NewTo + Normal);
        Points[2].x = X(theWindow, P3.x());
        Points[2].y = Y(theWindow, P3.y());
        Points[3].x = X(theWindow, P4.x());
        Points[3].y = Y(theWindow, P4.y());
        (*this) << GrFillPolygon(Points, 4, Convex, CoordModeOrigin, Color);
    }
}

/* =================================================================================== */

bool
TWindow::Dump(void)

{
    if (DumpActive)
    {
        char  Command[512];
        int   ExitCode;
        char *Home;
        char *PvmArch;
        char *Display;

        DumpCounter++;
        Home = getenv("HOME");
        PvmArch = getenv("PVM_ARCH");
        Display = getenv("DISPLAY");
        sprintf(Command,"%s/pvm3/bin/%s/dump %s %ld %s", Home, PvmArch,
                Display, theWindow->win, (char *) GetDumpLabel());
        Print("Executing command %s", Command);
        if ((ExitCode = (system(Command) >> 8)) != 0)
        {
            Error("system call returns %d", ExitCode);
            return FALSE;
        }
    }
    return TRUE;
}

/* =================================================================================== */

bool
TWindow::Execute(long ID)

{
    //    Print("Execute");
    if (!HandleXEvents(ID))
    {
        Error("Execute: while handling X-events");
        return FALSE;
    }

    if (Redraw())
        if (!Draw())
        {
            Error("Execute: while drawing");
            return FALSE;
        }
    if (!Display())
    {
        Error("Execute: while displaying");
        return FALSE;
    }
    return TRUE;
}

/* =================================================================================== */

TString
TWindow::GetClassName(void) const

{
    return "TWindow";
}

/* =================================================================================== */

TString
TWindow::GetDumpLabel(void) const

{
    char Label[256];

    sprintf(Label, "%s_%03d_TS%05ld", DumpBaseFilename(), DumpCounter, DumpID);
    return Label;
}

/* =================================================================================== */

bool
TWindow::HandleButtonPress(int x, int y, int ButtonState)

{
    Print("button %d pressed at (%d,%d)", ButtonState, x, y);
    return TRUE;
}

/* =================================================================================== */

bool
TWindow::HandleKeyPress(int KeyCode)

{
    switch(KeyCode)
    {
    case 'D':
        DumpOnNextRedraw();
        break;

    case 's':
        if (DumpActive)
        {
            DumpActive = FALSE;
            Print("GIF dump deactivated");
        }
        else
        {
            DumpActive = TRUE;
            Print("GIF dump activated");
        }
        break;
        
    default:
        Print("key %d pressed", KeyCode);       
    }
    return TRUE;
}

/* =================================================================================== */

bool
TWindow::HandleXEvents(long theDumpID)

{
    XEvent               Event;
    XButtonPressedEvent *BPEvent = (XButtonPressedEvent *) &Event;
    XKeyPressedEvent    *KPEvent = (XKeyPressedEvent *) &Event;
    char                 Buffer;
    KeySym               Symbol;
    XComposeStatus       ComposeStatus;

    DumpID = theDumpID;
    while (XCheckMaskEvent(X11Display(), EVENT_MASK, &Event))
    {
        switch (Event.type)
        {
        case Expose:
        case VisibilityNotify:
            MarkForRedisplay();
            break;

        case ButtonPress:
            if (!HandleButtonPress(BPEvent->x, BPEvent->y, BPEvent->button))
                return FALSE;
            break;

        case KeyPress:
            if (XLookupString(KPEvent, &Buffer, 1, &Symbol, &ComposeStatus) > 0)
                if (!HandleKeyPress((int) Symbol))
                    return FALSE;
            break;
        }
    }
    return TRUE;
}

/* =================================================================================== */

void
TWindow::_Init(void)

{
    RedrawFlag = TRUE;
    RedisplayFlag = TRUE;
    DumpCounter = 0;
    DumpActive = 0;
    DumpFlag = FALSE;
    DumpBaseFilename = "dump";
    InitialDeltaX = -1.0;
    InitialDeltaY = -1.0;
    if (Consistent())
    {
        CreateBackground(1);
        SetDrawable(1);
    }    
}

/* =================================================================================== */

void
TWindow::LabelDump(const char *Label)

{
    Warning("cannot label dump with '%s'", Label);
}

/* =================================================================================== */

void
TWindow::WriteCentered(REAL X, REAL Y, char *S, int Color)

{
    switch (HorizontalOverlap(X, S))
    {
    case GrLeftOverlap:
        SetHorizontalTextAlignment(GrLeftAlign);
        X = Xmin();
        break;
        
    case GrRightOverlap:
        SetHorizontalTextAlignment(GrRightAlign);
        X = Xmax();
        break;
        
    default:
        SetHorizontalTextAlignment(GrCenterAlign);
    }
    operator << (GrString(X, Y, S, Color));
}

/* =================================================================================== */
#undef ParentClass
