/*      file     : graphx/graphx.h
        project  : graphical toolbox
        author   : Michael Olesen
        comments : simple classes for using X11
        RCS      : $Id: graphx.h,v 1.4 2005/03/25 11:06:10 mr Exp $
*/

#ifndef __graphx__
#define __graphx__

#if defined(__cplusplus) || defined(c_plusplus)
extern "C"  {
#endif

#ifdef DEBUG

#ifdef VERSION
#undef VERSION
#endif

#define VERSION 1
#define SUBVERSION 99
#define RELEASE

#ifdef PACKAGE
#undef PACKAGE
#endif

#define PACKAGE "graphx (ANSI C)"

extern char grWhere[128];
#define SETFUNC(s) {strcpy(grWhere,s);\
  fprintf(stderr,"Entering: %s\n",grWhere);}
#define WARNING(s) fprintf(stderr,"WARNING: "\
  PACKAGE" version %d.%d.%d File: "__FILE__". Function: %s. %s.\n",\
  VERSION,SUBVERSION,RELEASE,grWhere,s)
#define ERROR(s) fprintf(stderr,"ERROR: "\
  PACKAGE" version %d.%d.%d File: "__FILE__". Function: %s. %s.\n",\
  VERSION,SUBVERSION,RELEASE,grWhere,s)
#define FATAL(s) {fprintf(stderr,"FATAL ERROR: "\
  PACKAGE" version %d.%d.%d File: "__FILE__". Function: %s. %s.\n",\
  VERSION,SUBVERSION,RELEASE,grWhere,s);exit(1);}
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#ifdef __hpux
#include <values.h>
#endif

#ifdef __hpux
#define __hpux_math_bug
#undef __cplusplus
#endif

#include <math.h>

#ifdef __hpux_math_bug
#define __cplusplus
#endif

#include <string.h>
#ifndef SUN4
/* #ifdef INCLUDE_XLIBINT */
#include <X11/Xlibint.h>
#endif
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <grFntDef.h>

#define GrStandardEventMask 0xFFFFFF

#define GR_MAX_COLORS      256
#define GR_MAX_BASE_COLORS 8

#define GR_RED             (-1)
#define GR_GREEN           (-2)
#define GR_BLUE            (-3)
#define GR_ORANGE          (-4)
#define GR_YELLOW          (-5)
#define GR_BROWN           (-6)
#define GR_VIOLET          (-7)
#define GR_CYAN            (-8)

#define GrXmin (-1)
#define GrXmax (1)
#define GrYmin (-1)
#define GrYmax (1)
#define GrZmin (-1)
#define GrZmax (1)

#define X(w,x) ((int)((x-w->Xmin)/(w->Xmax-w->Xmin)*(w->MAXX+1)))
#define Y(w,y) ((int)((1.0-(y-w->Ymin)/(w->Ymax-w->Ymin))*(w->MAXY+1)))
#define IX(w,x) ((x*(w->Xmax-w->Xmin)/(w->MAXX+1.0))+w->Xmin)
#define IY(w,y) ((1-((double)y)/((double)(w->MAXY+1.0)))*(w->Ymax-w->Ymin)+ \
  w->Ymin)
#define LX(w,lx) ((int)(fabs(lx/(w->Xmax-w->Xmin))*(w->MAXX+1)))
#define LY(w,ly) ((int)(fabs(ly/(w->Ymax-w->Ymin))*(w->MAXY+1)))
#define ILX(w,lx) (abs(lx)*(w->Xmax-w->Xmin)/(w->MAXX+1.0))
#define ILY(w,ly) (abs(ly)*(w->Ymax-w->Ymin)/(w->MAXY+1.0))
#define ANGLE(a) ((int)(a*64))

#ifndef min
#define min(a,b) ((a)<(b)?(a):(b))
#endif

#ifndef max
#define max(a,b) ((a)>(b)?(a):(b))
#endif

#ifndef AND
#define AND &&
#endif

#ifndef OR
#define OR ||
#endif

enum {GrFast,GrSlow};
enum {GrGrey,GrColor};

typedef struct grWindow_t grWindow;
typedef struct grFont_t grFont;
   
struct grWindow_t  {
  Display *dpy;
  unsigned scr,depth;
  Visual *vis;
  Window win,root;
  XSizeHints xsh;
  XSetWindowAttributes xswa;
  GC gc;
  XColor *colors;
  XEvent event;
  Drawable wd,*background;
  int noBackground;
  int speed;
  int MAXX,MAXY,noColors;
  int NrOfColorCells;
  int MaxColorIndex;
  int BaseColorIndex;
  double Xmin,Xmax,Ymin,Ymax,Zmin,Zmax;
  int theFont,theFontPS;
  int noFont,topFont;
  int horizontalTextAlignment,verticalTextAlignment;
  grFont **theFonts;
  char psFileName[80];
  int  psColor;
  FILE *psFile;
  int psLengthUnit,psPaperType;
};

extern char grFontFamilies[5][30];
extern char grFontWeights[2][30];
extern char grFontShapes[2][30];
extern char grFontNames[697][100];

grWindow *grCreateWindow(char *,int,int,int);
#define grConsistent(w) (w != (grWindow *)NULL)
void grDestroyWindow(grWindow *);

/*********************** Background Stuff ***************************/

void grCreateBackground(grWindow *,int);
#define grSetDrawable(w,n) (((n)<0 OR (n)>w->noBackground)?(0): \
  (w->wd=w->background[n],1))
int grBitBlt(grWindow *,int,int,int,int,int,int,int,int);
#define grUsrBitBlt(w,dst,dsx_x,dsx_y,src,src_x,src_y,width,height) \
  grBitBlt(w,dst,X(w,dst_x),Y(w,dst_y),src,X(w,src_x),Y(w,src_y), \
  LX(w,width),LX(w,height))
#define grDumpBackground(w,n) grBitBlt(w,0,0,0,n,0,0,w->MAXX+1,w->MAXY+1)

/*********************** Basic Setup Stuff **************************/

#define grSetTitle(w,wdNAME,iconNAME) XSetStandardProperties(w->dpy,w->win,\
  wdNAME,iconNAME,None,NULL,0,&(w->xsh))
#define grSetXaxis(w,xMin,xMax) {w->Xmin = xMin;w->Xmax = xMax;}
#define grSetYaxis(w,yMin,yMax) {w->Ymin = yMin;w->Ymax = yMax;}
#define grSetZaxis(w,zMin,zMax) {w->Zmin = zMin;w->Zmax = zMax;}

int
    grAllocateColorTransition(grWindow *theWindow,const char *FromColorName, 
                              const char *ToColorName, int NrOfSteps, 
                              int *BaseOffset);
void 
    grSetColorMode(grWindow *,int);

/**************************** Font Stuff ****************************/

#define GrDefaultNoFonts 10
enum {GrCenterAlign,GrLeftAlign,GrRightAlign,GrBottomAlign,GrTopAlign};
enum {grCourier,grHelvetica,grNewCenturySchoolbook,grTimes,grSymbol};
enum {grMedium,grBold};
enum {grRegular,grItalic};

int grLoadFont(grWindow *,int,int,int,int);
int grLoadFontName(grWindow *,char *);
void grUnloadFont(grWindow *,int,int,int,int);
void grUnloadFontName(grWindow *,char *);
#define grNoFonts(w) (w->noFont)
#define grTopFont(w) (w->topFont)
int grGetFontNo(grWindow *,int,int,int,int);
int grSetFont(grWindow *,int,int,int,int);
#define grSetHorizontalTextAlignment(w,a) ((a!=GrCenterAlign AND \
  a!=GrLeftAlign AND a!=GrRightAlign)?(0):(w->horizontalTextAlignment= (int) a,1))
#define grGetHorizontalTextAlignment(w) (w->horizontalTextAlignment)
#define grSetVerticalTextAlignment(w,a) ((a!=GrCenterAlign AND \
  a!=GrBottomAlign AND a!=GrTopAlign)?(0):(w->verticalTextAlignment= (int) a,1))
#define grGetVerticalTextAlignment(w) (w->verticalTextAlignment)

#define grGetTextWidth(w,s) (XTextWidth(w->theFonts[w->theFont]->theFont, \
  s,strlen(s)))
#define grGetTextHeight(w) (w->theFonts[w->theFont]->theFont->ascent+ \
  w->theFonts[w->theFont]->theFont->descent)
#define grUsrGetTextWidth(w,s) ILX(grGetTextWidth(w,s))
#define grUsrGetTextHeight(w) ILY(grGetTextHeight(w))
  
enum {GrNoOverlap,GrLeftOverlap,GrRightOverlap,GrBottomOverlap,GrTopOverlap};

#define grLeftTextBoundary(w,x,s) ((grGetHorizontalTextAlignment(w) == \
  GrLeftAlign)?(x):((grGetHorizontalTextAlignment(w) == GrCenterAlign)? \
  (x-grGetTextWidth(w,s)/2):(x-grGetTextWidth(w,s))))
#define grRightTextBoundary(w,x,s) ((grGetHorizontalTextAlignment(w) == \
  GrLeftAlign)?(x+grGetTextWidth(w,s)):((grGetHorizontalTextAlignment(w) == \
  GrCenterAlign)?(x+grGetTextWidth(w,s)/2):(x)))
#define grTopTextBoundary(w,y,s) ((grGetVerticalTextAlignment(w) == \
  GrTopAlign)?(y):((grGetVerticalTextAlignment(w) == GrCenterAlign)? \
  (y-grGetTextHeight(w)/2):(y-grGetTextHeight(w))))
#define grBottomTextBoundary(w,y,s) ((grGetVerticalTextAlignment(w) == \
  GrTopAlign)?(y+grGetTextHeight(w)):((grGetVerticalTextAlignment(w) == \
  GrCenterAlign)?(y+grGetTextHeight(w)/2):(y)))

#define grUsrLeftTextBoundary(w,x,s) IX(w,grLeftTextBoundary(w,X(w,x),s))
#define grUsrRightTextBoundary(w,x,s) IX(w,grRightTextBoundary(w,X(w,x),s))
#define grUsrTopTextBoundary(w,y,s) IY(w,grTopTextBoundary(w,Y(w,y),s))
#define grUsrBottomTextBoundary(w,y,s) IY(w,grBottomTextBoundary(w,Y(w,y),s))

#define grHorizontalOverlap(w,x,s) ((grLeftTextBoundary(w,x,s) < 0)? \
  (GrLeftOverlap):((grRightTextBoundary(w,x,s) > w->MAXX)? \
  (GrRightOverlap):(GrNoOverlap)))
#define grVerticalOverlap(w,y,s) ((grTopTextBoundary(w,y,s) < 0)? \
  (GrTopOverlap):((grBottomTextBoundary(w,y,s) > w->MAXY)? \
  (GrBottomOverlap):(GrNoOverlap)))
  
#define grUsrHorizontalOverlap(w,x,s) grHorizontalOverlap(w,X(w,x),s)
#define grUsrVerticalOverlap(w,y,s) grVerticalOverlap(w,Y(w,y),s)

#define grHorizontalAdjustTextAlignment(w,x,s) ((grHorizontalOverlap(w,x,s) == \
  GrLeftOverlap)?(grSetHorizontalTextAlignment(w,GrLeftAlign)): \
  ((grHorizontalOverlap(w,x,s) == GrRightOverlap)? \
  (grSetHorizontalTextAlignment(w,GrRightAlign)):(True)))
#define grVerticalAdjustTextAlignment(w,y,s) ((grVerticalOverlap(w,y,s) == \
  GrTopOverlap)?(grSetVerticalTextAlignment(w,GrTopAlign)): \
  ((grVerticalOverlap(w,y,s) == GrBottomOverlap)? \
  (grSetVerticalTextAlignment(w,GrBottomAlign)):(True)))

#define grUsrHorizontalAdjustTextAlignment(w,x,s) \
  grHorizontalAdjustTextAlignment(w,X(w,x),s)
#define grUsrVerticalAdjustTextAlignment(w,y,s) \
  grVerticalAdjustTextAlignment(w,Y(w,y),s)

#define grAdjustTextAlignment(w,x,y,s) {grHorizontalAdjustTextAlignment(w,x,s); \
  grVerticalAdjustTextAlignment(w,y,s);}
  
#define grUsrAdjustTextAlignment(w,x,y,s) grAdjustTextAlignment(w,X(w,x), \
  Y(w,y),s)
  
void grDrawChar(grWindow *,int,int,char,int);
void grDrawString(grWindow *,int,int,char *,int);
void grDrawFillChar(grWindow *,int,int,char,int,int);
void grDrawFillString(grWindow *,int,int,char *,int,int);

#define grUsrDrawChar(w,x,y,c,color) grDrawChar(w,X(w,x),Y(w,y),c,color)
#define grUsrDrawString(w,x,y,s,color) grDrawString(w,X(w,x),Y(w,y),s,color)
#define grUsrDrawFillChar(w,x,y,c,fg,bg) grDrawFillChar(w,X(w,x),Y(w,y),c,fg,bg)
#define grUsrDrawFillString(w,x,y,s,fg,bg) grDrawFillString(w,X(w,x),Y(w,y),s,fg,bg)

/********************** Drawing Primitives **************************/

void grClearScreen(grWindow *,int);

void grDrawPoint(grWindow *,int,int,int);
void grDrawLine(grWindow *,int,int,int,int,int);
void grDrawBox(grWindow *,int,int,int,int,int);
void grDrawFillBox(grWindow *,int,int,int,int,int);
void grDrawArc(grWindow *,int,int,int,double,double,int);
void grDrawFillArc(grWindow *,int,int,int,double,double,int);
void grDrawCircle(grWindow *,int,int,int,int);
void grDrawFillCircle(grWindow *,int,int,int,int);
void grDrawEllipticArc(grWindow *,int,int,int,int,double,double,int);
void grDrawFillEllipticArc(grWindow *,int,int,int,int,double,double,int);
void grDrawEllipse(grWindow *,int,int,int,int,int);
void grDrawFillEllipse(grWindow *,int,int,int,int,int);
void grDrawFillPolygon(grWindow *,XPoint *, int,int,int,int);

#define grUsrDrawPoint(w,x,y,c) grDrawPoint(w,X(w,x),Y(w,y),c)
#define grUsrDrawLine(w,x1,y1,x2,y2,c) grDrawLine(w,X(w,x1),Y(w,y1), \
  X(w,x2),Y(w,y2),c)
#define grUsrDrawBox(w,x1,y1,x2,y2,c) grDrawBox(w,X(w,x1),Y(w,y1), \
  X(w,x2),Y(w,y2),c)
#define grUsrDrawFillBox(w,x1,y1,x2,y2,c) grDrawFillBox(w,X(w,x1),Y(w,y1), \
  X(w,x2),Y(w,y2),c)
#define grUsrDrawArc(w,x,y,r,a1,a2,c) grDrawEllipticArc(w,X(w,x),Y(w,y), \
  LX(w,r),LY(w,r),a1,a2,c)
#define grUsrDrawFillArc(w,x,y,r,a1,a2,c) grDrawFillEllipticArc(w,X(w,x),Y(w,y), \
  LX(w,r),LY(w,r),a1,a2,c)
#define grUsrDrawCircle(w,x,y,r,c) grDrawEllipse(w,X(w,x),Y(w,y), \
  LX(w,r),LY(w,r),c)
#define grUsrDrawFillCircle(w,x,y,r,c) grDrawFillEllipse(w,X(w,x),Y(w,y), \
  LX(w,r),LY(w,r),c)
#define grUsrDrawEllipticArc(w,x,y,a,b,a1,a2,c) grDrawEllipticArc(w, \
  X(w,x),Y(w,y),LX(w,a),LY(w,b),a1,a2,c)
#define grUsrDrawFillEllipticArc(w,x,y,a,b,a1,a2,c) grDrawFillEllipticArc(w, \
  X(w,x),Y(w,y),LX(w,a),LY(w,b),a1,a2,c)
#define grUsrDrawEllipse(w,x,y,a,b,c) grDrawEllipse(w,X(w,x),Y(w,y), \
  LX(w,a),LY(w,b),c)
#define grUsrDrawFillEllipse(w,x,y,a,b,c) grDrawFillEllipse(w,X(w,x),Y(w,y), \
  LX(w,a),LY(w,b),c)

/**************************** X11 Stuff ****************************/

#define grX11Display(w) (w->dpy)
#define grX11Screen(w) (w->scr)
#define grX11Depth(w) (w->depth)
#define grX11Visual(w) (w->vis)
#define grX11GC(w) (w->gc)
#define grX11RootWindow(w) (w->root)
#define grX11Window(w) (w->win)
#define grX11XSizeHints(w) (w->xsh)
#define grX11XSetWindowAttributes(w) (w->xswa)
#define grX11XColor(w,n) (w->colors[(n)%w->noColors])
#define grX11XEvent(w) (w->event)

#define grX11CheckMaskEvent(w,eventMask) (XCheckMaskEvent(w->dpy,eventMask, \
  &(w->event)))
#define grX11NextEvent(w) (XNextEvent(w->dpy,&(w->event)))
#define grX11WaitUntilExposed(w) while(!grX11CheckMaskEvent(w,ExposureMask))
#define grX11WaitUntilUnmapped(w) while(grX11CheckMaskEvent(w,0xFFFFFFFF))

#define grX11SetSpeed(w,s) (w->speed = s)
#define grX11GetSpeed(w) (w->speed)
#define grX11Flush(w) (XFlush(w->dpy))

#define grX11SelectInput(w,inputMask) (XSelectInput(w->dpy,w->win,inputMask))

#define grX11SetCursor(w,cursor) \
  (XDefineCursor(w->dpy,w->win,XCreateFontCursor(w->dpy,cursor)))

/************************* PostScript Stuff *************************/

#define GrUnitTypes 2
enum {GrInch,GrMm};
extern double GrUnitLengths[GrUnitTypes];

#define GrPaperTypes 2
enum {GrA4,GrUSletter};
extern double GrPaperDimensions[GrPaperTypes][GrUnitTypes][2];

#define PSLENX(w) (GrPaperDimensions[w->psPaperType][w->psLengthUnit][0]*\
  GrUnitLengths[w->psLengthUnit])
#define PSLENY(w) (GrPaperDimensions[w->psPaperType][w->psLengthUnit][1]*\
  GrUnitLengths[w->psLengthUnit])
#define PSSCALE(w) min(PSLENX(w)/(w->MAXX+1.0),PSLENY(w)/(w->MAXY+1.0))
#define PSX(w,x) ((x-w->Xmin)/(w->Xmax-w->Xmin)*PSSCALE(w)*(w->MAXX+1.0))
#define PSY(w,y) ((y-w->Ymin)/(w->Ymax-w->Ymin)*PSSCALE(w)*(w->MAXY+1.0))
#define PSLX(w,lx) (fabs(lx/(w->Xmax-w->Xmin)*PSSCALE(w)*(w->MAXX+1.0)))
#define PSLY(w,ly) (fabs(ly/(w->Ymax-w->Ymin)*PSSCALE(w)*(w->MAXY+1.0)))

void grOpenPostScript(grWindow *,char *);
void grClosePostScript(grWindow *);

void grSetLengthUnit(grWindow *,int);
#define grGetLengthUnit(w) (w->psLengthUnit)
void grSetPaperType(grWindow *,int);
#define grGetPaperType(w) (w->psPaperType)

void grClearScreenPS(grWindow *,int);

void grDrawPointPS(grWindow *,double,double,int); /* undone */
void grDrawLinePS(grWindow *,double,double,double,double,int);
void grDrawBoxPS(grWindow *,double,double,double,double,int);
void grDrawFillBoxPS(grWindow *,double,double,double,double,int);
void grDrawArcPS(grWindow *,double,double,double,double,double,int);
void grDrawFillArcPS(grWindow *,double,double,double,double,double,int);
void grDrawCirclePS(grWindow *,double,double,double,int);
void grDrawFillCirclePS(grWindow *,double,double,double,int);
void grDrawEllipticArcPS(grWindow *,double,double,double,double,
  double,double,int);
void grDrawFillEllipticArcPS(grWindow *,double,double,double,double,
  double,double,int);
void grDrawEllipsePS(grWindow *,double,double,double,double,int);
void grDrawFillEllipsePS(grWindow *,double,double,double,double,int);
void grSetGreyPS(grWindow *, int); /* MR */

/*************************** Info Stuff *****************************/

#define grNumColors(w) (w->noColors)
#define grMaxX(w) (w->MAXX)
#define grMaxY(w) (w->MAXY)
#define grXmin(w) (w->Xmin)
#define grXmax(w) (w->Xmax)
#define grYmin(w) (w->Ymin)
#define grYmax(w) (w->Ymax)
#define grZmin(w) (w->Zmin)
#define grZmax(w) (w->Xmax)

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif
