/*      file     : mpi/TMPIBuffer.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TMPIBuffer.C,v 1.5 2005/05/01 14:57:58 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include "TMPIBuffer.h"

// ***DEFINES************************************************************************* 

#define MPI_MAX_SAVE_BUFFER_USAGE 0.8

// ***GLOBAL VARIABLES****************************************************************

TDLList<TMPIBuffer> MPIFreeBuffers;
TDLList<TMPIBuffer> MPIUsedBuffers;

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#  include "TMPIBuffer.i.C"
#endif

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 


TMPIBuffer::TMPIBuffer(long theSize)

{
    Size = theSize;
    User = 0;
    Active = FALSE;
    if ((Base = new char[Size]) == 0)
    {
        Fail("out of heap");
    }
}

// =================================================================================== 

TMPIBuffer::~TMPIBuffer(void)

{
    if (Active)
    {
        Warning("~TMPIBuffer: buffer still used by (%s)",
                (char *) User->GetInstanceName());
        MPIUsedBuffers.Delete(this);
    }
    else
        MPIFreeBuffers.Delete(this);
    DeleteObjectArray(Base);
}

// =================================================================================== 

void
TMPIBuffer::Activate(TBaseObject *theUser)

{
    if (Active)
    {
        Error("Activate: buffer already used by (%s)", (char *) User->GetInstanceName());
        return;
    }
    User = theUser;
    MPIFreeBuffers.Delete(this);
    MPIUsedBuffers.Append(this);
    Position = 0;
    Active = TRUE;
}

// ===================================================================================

void
TMPIBuffer::Deactivate(void)

{
    if (!Active)
    {
        Warning("Deactivate: buffer was not active");
        return;
    }
    if (Position > (long) (MPI_MAX_SAVE_BUFFER_USAGE * (double) Size))
        Warning("Deactivate: used %d/%d bytes", Position, Size);
    MPIUsedBuffers.Delete(this);
    MPIFreeBuffers.Append(this);
    User = 0;
    Active = FALSE;
}

// ===================================================================================

TString
TMPIBuffer::GetClassName(void) const

{
    return "MPIBuffer";
}

// =================================================================================== 
#undef ParentClass


