/*      file     : parallel/TCPNInfo.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TCPNInfo.C,v 1.7 2005/03/25 11:06:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include "TBaseMessage.h"
#include "TCPNInfo.h"

// ***DEFINES************************************************************************* 

#define CPNINFO_DEFAULT_FLAGS 0

// ***FUNCTIONS*********************************************************************** 

int
CompareCPNInfosByHostAndID(const TCPNInfo **Info1, const TCPNInfo **Info2)

{
    if ((*Info1)->HostID > (*Info2)->HostID)
        return 1;
    else
        if ((*Info1)->HostID < (*Info2)->HostID)
            return -1;
        else
            return CompareCPNInfosByID(Info1, Info2);
}

// =================================================================================== 

int
CompareCPNInfosByID(const TCPNInfo **Info1, const TCPNInfo **Info2)

{
    if ((*Info1)->ID > (*Info2)->ID)
        return 1;
    else
        if ((*Info1)->ID < (*Info2)->ID)
            return -1;
        else
            return 0;
}


// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#endif

// =================================================================================== 

// =================================================================================== 
#define ParentClass
// =================================================================================== 

TCPNInfo::TCPNInfo(void)

{
    _Init();
}

// =================================================================================== 

TCPNInfo::TCPNInfo(TBaseMessage *aMessage)

{
    _Init();
    if (!Decode(aMessage))
        Fail("cannot decode from message");
}

// =================================================================================== 

TCPNInfo::TCPNInfo(const TCPNInfo *aCPNInfo)

{
    ID = aCPNInfo->ID;
    HostID = aCPNInfo->HostID;
    Name = aCPNInfo->Name;
    Flags = aCPNInfo->Flags;    
    Performance = aCPNInfo->Performance;
}

// =================================================================================== 

void
TCPNInfo::operator = (TCPNInfo aCPNInfo)

{
    ID = aCPNInfo.ID;
    HostID = aCPNInfo.HostID;
    Name = aCPNInfo.Name;
    Flags = aCPNInfo.Flags;
    Performance = aCPNInfo.Performance;
}

// =================================================================================== 

bool
TCPNInfo::operator == (const TCPNInfo &aCPNInfo) const

{
    return (aCPNInfo.ID == ID);
}

// =================================================================================== 

bool
TCPNInfo::Decode(TBaseMessage *aMessage)

{
    return (aMessage->Decode(ID) AND
            aMessage->Decode(HostID) AND
            Name.Decode(*aMessage) AND
            aMessage->Decode(DebugMask) AND
            aMessage->Decode(Performance) AND
            aMessage->Decode(Flags));
}

// =================================================================================== 

bool
TCPNInfo::Encode(TBaseMessage *aMessage) const

{
    return (aMessage->Encode(ID) AND
            aMessage->Encode(HostID) AND
            Name.Encode(*aMessage) AND
            aMessage->Encode(DebugMask) AND
            aMessage->Encode(Performance) AND       
            aMessage->Encode(Flags));
}

// =================================================================================== 

TString
TCPNInfo::GetClassName(void) const

{
    return "TCPNInfo";
}

// =================================================================================== 

TString
TCPNInfo::GetInstanceName(void) const

{
    char Line[512];

    sprintf(Line, "ID:%lX HostID:%lX Name:'%s'",
            ID, HostID, Name());
    return Line;
}

// =================================================================================== 

void
TCPNInfo::_Init(void)

{
    ID = INVALID_ID;
    HostID = INVALID_ID;
    Name = "unknown";
    Flags = CPNINFO_DEFAULT_FLAGS;
}

// =================================================================================== 
#undef ParentClass
