/*      file     : parallel/TDestination.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TDestination.h,v 1.8 2005/03/25 11:06:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TDESTINATION_H
#define TDESTINATION_H

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>
#include <TDLList.h>

#include "ParallelTypes.h"

// ***FORWARDS************************************************************************

class TCPNInfo;

// ***DEFINES************************************************************************* 

#define DEST_LIST            (-2)
#define DEST_ALL             (-3)
#define DEST_ALL_BUT_MYSELF  (-4)
#define DEST_MYSELF          (-5)
#define DEST_PARENT          (-6)
#define DEST_NEIGHBORS       (-7)

#define SOURCE_LIST            DEST_LIST
#define SOURCE_ALL             DEST_ALL
#define SOURCE_ALL_BUT_MYSELF  DEST_ALL_BUT_MYSELF
#define SOURCE_MYSELF          DEST_MYSELF

#define DEST_FLAG_EXCLUDE_PARENT   0x00000001
#define DEST_FLAG_EXCLUDE_MYSELF   0x00000002
#define DEST_FLAG_INCLUDES_PARENT  0x00000004
#define DEST_FLAG_INCLUDES_MYSELF  0x00000008

// ***TYPES*************************************************************************** 

typedef long TDestFlags;

struct TDestMutable

{
    TDestFlags  Flags;
    long        NrOfRecipients;
    TCPNID     *Recipients;
};

// ***CLASSES************************************************************************* 

class TDestination : public TBaseObject

{
 public:

    TDestination(void);
    TDestination(const TDestination &aDestination);
    TDestination(TCPNID theRecipient);
    TDestination(const TCPNInfo *theRecipient);
    TDestination(const TDLList<TCPNInfo> *theCPNInfoList, TDestFlags theFlags = 0);
    virtual ~TDestination(void);

    inline TDestFlags
        GetFlags(void) const;
    virtual TString
        GetInstanceName(void) const;
    inline long
        GetNrOfRecipients(void) const;
    inline TCPNID
        GetRecipient(void) const;
    inline const TCPNID *
        GetRecipients(void) const;
    bool
        IsRecipient(TCPNID aRecipient) const;

 private:

    bool
        CheckRecipients(void) const;
    virtual TString
        GetClassName(void) const;
    void
        _Init(void);

    TCPNID                   Recipient;
    const TDLList<TCPNInfo> *CPNInfoList;
    TDestMutable            *Mutable;
};

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TDestination.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
