/*      file     : parallel/TEventHandler.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        objects  : TEventHandlerPrimitive
                   TEventHandler<>
        RCS      : $Id: TEventHandler.h,v 1.7 2005/03/25 11:06:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TEVENTHANDLER_H
#define TEVENTHANDLER_H

#include <gcc_pragma_interface.h>

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <CommonTypes.h>
#include <ParallelTypes.h>
#include <TBaseObject.h>
#include <TString.h>

// ***FORWARDS************************************************************************

class TBaseMessage;

// ***CLASSES************************************************************************* 

class TEventHandlerPrimitive : public TBaseObject

{
public:

    TEventHandlerPrimitive(TEventHandlerPrimitive *theRelayHandler = 0);
    virtual ~TEventHandlerPrimitive(void);

    virtual bool
        HandleEvent(TEventID EventID,  TBaseMessage *Message, int &Handled) const = 0;
    bool
        RegisterEventHandler(TEventID EventID, TEventHandlerPrimitive *theHandler);

protected:

    TEventHandlerPrimitive  *RelayHandler;
    TEventHandlerPrimitive **HandlerObjects;
};

// ===================================================================================

template <class TReceiver>
class TEventHandler : public TEventHandlerPrimitive

{
public:

    TEventHandler(TReceiver *theLocalHandler, TEventHandlerPrimitive *theRelayHandler = 0);
    virtual ~TEventHandler(void);

    virtual bool
        HandleEvent(TEventID EventID, TBaseMessage *Message, int &Handled) const;
    bool
        Register(TEventID EventID,  bool (TReceiver::*EventMethod) (TBaseMessage *, int &));

private:

    typedef bool (TReceiver::*PEventMethod) (TBaseMessage *, int &);

    PEventMethod *EventMethods;
    TReceiver    *LocalHandler;

    virtual TString
        GetClassName(void) const;
};

// ***TEMPLATE METHODS****************************************************************

#if defined(INCLUDE_TEMPLATES) || defined(TEMPLATEBODIES)
#include "TEventHandler.t.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
