/*      file     : parallel/TGridTopology.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TGridTopology.C,v 1.3 2005/03/25 11:06:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdlib.h>

#include "TParallel.h"
#include "TCPNInfo.h"

#include "TGridTopology.h"

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#  include "TGridTopology.i.C"
#endif

// =================================================================================== 
#define ParentClass TTopology
// =================================================================================== 

TGridTopology::TGridTopology(int theRows, int theCols, const TDLList<TCPNInfo> *theList) :

TTopology(theList)

{
    GridMutable = new TGridTopologyMutable;
    GridMutable->Rows = theRows;
    GridMutable->Cols = theCols;
    GridMutable->RowList = 0;
    GridMutable->ColList = 0;
}

// ===================================================================================

TGridTopology::~TGridTopology(void)

{
    DeleteObject(GridMutable->RowList);
    DeleteObject(GridMutable->ColList);
    DeleteObject(GridMutable);
}

// =================================================================================== 

bool
TGridTopology::BuildRowAndColumnLists(void)

{
    if (GridMutable->RowList == 0)
    {
        int Row;
        
        GridMutable->RowList = new TDLList < TCPNInfo >;

        for (Row = 0; Row < GetNrOfRows(); Row++)
            GridMutable->RowList->Append(GetCPNInfoByRowAndCol(Row, GetMyCol()));
    }
    if (GridMutable->ColList == 0)
    {
        int Col;
        
        GridMutable->ColList = new TDLList < TCPNInfo >;

        for (Col = 0; Col < GetNrOfCols(); Col++)
            GridMutable->ColList->Append(GetCPNInfoByRowAndCol(GetMyRow(), Col));
    }
    return TRUE;
}

// ===================================================================================

TString
TGridTopology::GetClassName(void) const

{
    return "TGridTopology";
}

// =================================================================================== 

TString
TGridTopology::GetInstanceName(void) const

{
    char Line[256];

    sprintf(Line, "%dx%d", GetNrOfRows(), GetNrOfCols());
    return Line;
}

// =================================================================================== 

TTopologyType
TGridTopology::GetType(void) const

{
    return GridTopology;
}

// =================================================================================== 
#undef ParentClass
