/*      file     : parallel/TGridTopology.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TGridTopology.h,v 1.5 2005/03/25 11:06:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TGRIDTOPOLOGY_H
#define TGRIDTOPOLOGY_H

// ***INCLUDES************************************************************************ 

#include <TDLList.h>
#include <TObjectArray.h>

#include "TTopology.h"

// ***CLASSES************************************************************************* 

struct TGridTopologyMutable

{
    int Rows;
    int Cols;
    TDLList< TCPNInfo > *RowList;
    TDLList< TCPNInfo > *ColList;
};

// ===================================================================================

/** Topology class representing a two-dimensional grid.
  */

class TGridTopology : public TTopology

{
public:

    TGridTopology(int theRows, int theCols, const TDLList<TCPNInfo> *theList = 0);
    virtual ~TGridTopology(void);

    inline TDLList< TCPNInfo > *
        GetColList(void);
    inline const TCPNInfo *
        GetCPNInfoByRowAndCol(int theRow, int theCol) const;
    inline int
        GetMyCol(void) const;
    inline int
        GetMyRow(void) const;
    inline int
        GetNrOfCols(void) const;
    inline int
        GetNrOfRows(void) const;
    inline int
        GetRankByRowAndCol(int theRow, int theCol) const;
    inline TDLList< TCPNInfo > *
        GetRowList(void);
    virtual TTopologyType
        GetType(void) const;

private:

    bool
        BuildRowAndColumnLists(void);
    virtual TString
        GetClassName(void) const;
    virtual TString
        GetInstanceName(void) const;

    TGridTopologyMutable *GridMutable;
};

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TGridTopology.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
