/*      file     : parallel/TParallelControl.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TParallelControl.h,v 1.25 2005/03/25 11:06:21 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TPARALLELCONTROL_H
#define TPARALLELCONTROL_H

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>
#include <TObjectArray.h>
#include <TDLList.h>
#include <TMessageRecipient.h>
#include <TTopologyUser.h>
#include <TDestination.h>
#include <TEventHandler.h>

// ***FORWARDS************************************************************************

#include <TCPNInfo.forward.h>
#include <THostInfo.forward.h>
class TTopology;

// ***DEFINES************************************************************************* 

#define EVENT_CPNINFO            1
#define EVENT_SYNC_REQ           2
#define EVENT_SYNC_REQ_EXIT_LOOP 3
#define EVENT_SYNC_ACK           4
#define EVENT_QUIT               5

#define PC_FLAG_SLAVES_NEED_PRINT_PREFIX 1

// ***TYPES*************************************************************************** 

struct TParallelMutable

{
    TCPNInfo           *ParentCPNInfo;
    TCPNInfo           *LocalCPNInfo;
    THostInfo          *LocalHostInfo;
    TDLList<TCPNInfo>  *CPNInfoList;
    TDLList<THostInfo> *HostInfoList;
    TTopology          *PhysicalTopology;
};

// ***CLASSES************************************************************************* 

/** Low level parallel control class.
  */

class TParallelControl : public TBaseObject, public TMessageRecipient

{
public:

    TParallelControl(void);
    virtual ~TParallelControl(void);

    virtual bool
        Activate(void);
    virtual bool
        AddCPN(THostInfo *aHostInfo, int &NrOfInstances);
    virtual bool
        Disenroll(void);
    virtual bool
        Enroll(void);
    TCPNInfo *
        GetCPNInfoByID(TCPNID SearchID) const;
    virtual const TDLList<TCPNInfo> *
        GetCPNInfoList(void) const;
    int
        GetCurrentNrOfCPN(void) const;
    int
        GetCurrentNrOfHosts(void) const;
    virtual const TDLList<THostInfo> *
        GetHostInfoList(void) const;
    virtual TString
        GetInstanceName(void) const;
    TCPNID
        GetLocalCPNID(void) const;
    const TCPNInfo *
        GetLocalCPNInfo(void) const;
    const THostInfo *
        GetLocalHostInfo(void) const;
    virtual int
        GetMaxNrOfCPN(void) const;
    virtual TCPNID
        GetParentCPNID(void) const;
    TTopology *
        GetPhysicalTopology(void) const;
    inline long
    GetUniqueID(long OtherID);
    inline bool
        HasQuit(void) const;
    inline bool
        IsEnrolled(void) const;
    virtual bool
        IsMaster(void) const;
    bool
        Quit(void) const;
    inline bool
        RegisterTopologyUser(TTopologyUser *theUser);
    bool
        SendChangesInTopology(void);
    inline bool
        SlavesNeedPrintPrefix(void) const;
    bool
        Sync(void);
    bool
        Synchronize(TDestination Destination, bool ExitLoop = FALSE);

protected:
    
    virtual bool
        CheckCPNInfoList(void) const = 0;
    virtual bool
        CheckHostInfoList(void) const = 0;
    TCPNInfo *
        GetCPNInfo(THostInfo *aHostInfo, TCPNID theCPNID) const;

    TParallelMutable   *Mutable;

private:

    bool
        AddCPNInfo(const TCPNInfo *aCPNInfo);
    void
        DetermineBestGrid(int &Rows, int &Cols, int Count, int &Remainder) const;
    bool
        DeterminePhysicalTopology(void) const;
    bool
        HandleEventCPNInfo(TBaseMessage *aMessage, int &Handled);
    bool
        HandleEventQuit(TBaseMessage *, int &Handled);
    bool
        HandleEventSyncAck(TBaseMessage *theMessage, int &Handled);
    bool
        HandleEventSyncReq(TBaseMessage *theMessage, int &Handled);
    bool
        HandleEventSyncReqExitLoop(TBaseMessage *theMessage, int &Handled);
    void
        _Init(void);

    TArray<TTopologyUser *>          TopologyUsers;
    TEventHandler<TParallelControl> *EventHandler;

    long   Flags;
    long   SyncAckMissing;
    bool   Enrolled;
    bool   QuitFlag;
};

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TParallelControl.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
