/*      file     : TrafficData/TAsciiObjectTimeStatReader.C
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TAsciiObjectTimeStatReader.C,v 1.3 2005/04/01 15:11:25 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include "TObjectTimeStat.h"

#include "TAsciiObjectTimeStatReader.h"

// ***DEFINES************************************************************************* 

#define OBJECTTIMESTAT_ASCII_TABLE_NAME "ObjectTimers"
#define OBJECTTIMESTAT_SCAN_PATTERN     "%lld"

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TAsciiDataTypeReader
// =================================================================================== 

TAsciiObjectTimeStatReader::TAsciiObjectTimeStatReader(TClassInfo *theClassInfo,
                                                 const char *theBaseFilename) :

ParentClass(theBaseFilename, OBJECTTIMESTAT_ASCII_TABLE_NAME, OBJECTTIMESTAT_SCAN_PATTERN)

{    
    ClassInfo = theClassInfo;
}

// =================================================================================== 

void *
TAsciiObjectTimeStatReader::CreateObject(const char *Line) const

{
    TObjectTimeStat *ObjectTimeStat = 0;
    TParallelID      ParallelID;
    double           Time;

    if (sscanf(Line, "%lld %lf", &ParallelID, &Time) != 2)
    {
        Error("too few parameters in line '%s'", Line);
        return 0;
    }
    if ((ObjectTimeStat = new TObjectTimeStat(ClassInfo, ParallelID, Time)) == 0)
    {
        Error("cannot create TObjectTimeStat");
        return 0;
    }
    return ObjectTimeStat;
}

// =================================================================================== 

void 
TAsciiObjectTimeStatReader::DisposeObject(void *Object)

{
    delete (TObjectTimeStat *) Object;
}

// =================================================================================== 
#undef ParentClass
