/*      file     : parstat/TAggregationInfo.C
        project  : Parallel Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TAggregationInfo.C,v 1.4 2005/03/25 11:08:01 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <TTopology.h>

#include "TTimeStep.h"
#include "TSamplingDescription.h"
#include "TStatObject.h"
#include "TAggregationResult.h"
#include "TAggregationInfo.h"

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

TAggregationInfo::TAggregationInfo(TStatHandler *theResultHandler,
                                   TSamplingDescription *SamplingDescription)

{
    _Init();
    ResultHandler = theResultHandler;
    SamplingID = SamplingDescription->GetParallelID();
    Results = new TAggregationResultTable;
}

// =================================================================================== 

TAggregationInfo::TAggregationInfo(TParallelID theSamplingID)

{
    _Init();
    SamplingID = theSamplingID;
}

// =================================================================================== 

TAggregationInfo::~TAggregationInfo(void)

{
    if (Results)
    {
        TEnumerator< TAggregationResult > CResult(Results);
        int Count;

        Count = 0;
        CResult.Enumerate();
        while (CResult())
        {
            if (!CResult->Handled)
                Count++;
            CResult++;
        }
        if (Count)
            Warning("DESTRUCTOR: %d stat results not handled", Count);
        DeleteObject(Results);
    }
}

// =================================================================================== 

int
TAggregationInfo::Compare(TBaseObject *OtherObject, int) const

{
    TAggregationInfo *OtherInfo = (TAggregationInfo *) OtherObject;

    return CompareLong(OtherInfo->SamplingID, SamplingID);
}

// =================================================================================== 

void
TAggregationInfo::DisposeResult(TTimeStep &TimeStep)

{
    TTimedAggregationResult *Entry;
    TAggregationResult      *Result;

    
    if ((Entry = Results->Delete(TimeStep)) == 0)
    {
        Warning("DisposeResult: cannot dispose result for time-step %s",
                (char *) TimeStep.GetInstanceName());
        return;
    }
    Result = *Entry;
    DeleteObject(Result);
    DeleteObject(Entry);
}

// =================================================================================== 

TString
TAggregationInfo::GetClassName(void) const

{
    return "TAggregationInfo";
}

// =================================================================================== 

TStatHandler *
TAggregationInfo::GetHandler(void) 

{
    return ResultHandler;
}

// =================================================================================== 

TString
TAggregationInfo::GetInstanceName(void) const

{
   char Line[256];

   sprintf(Line, "aggregating SamplingID %ld", SamplingID);
   return Line;
}

// =================================================================================== 

TAggregationResult *
TAggregationInfo::GetResult(TTimeStep &TimeStep, int NrOfCPN)

{
    TTimedAggregationResult *Entry = 0;
    TAggregationResult      *Result = 0;

    if ((Entry = Results->Find(TimeStep)) == 0)
    {
        if (NrOfCPN == INVALID)
        {
            Warning("GetResult: no result info found for time-step %s",
                    (char *) TimeStep.GetInstanceName());
            return 0;
        }
        Result = new TAggregationResult(NrOfCPN);
        Entry = new TTimedAggregationResult(TimeStep, Result);
        if (!Results->Insert(Entry, TRUE))
        {
            Error("GetResult: while inserting new result info");
            return 0;
        }
        Print(DM_STAT_AGGR, "GetResult: created new info for time-step %s",
              (char *) TimeStep.GetInstanceName());
    }
    return *Entry;
}

// =================================================================================== 

void
TAggregationInfo::_Init(void)

{
    Results = 0;
}

// =================================================================================== 
#undef ParentClass
