/*      file     : parstat/TAggregationInfo.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TAggregationInfo.h,v 1.5 2005/03/25 11:08:01 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TAGGREGATIONINFO_H
#define TAGGREGATIONINFO_H

// ***INCLUDES************************************************************************ 

#include <ParallelTypes.h>
#include <TParallelObject.h>
#include <TDLList.h>
#include <TABTree.h>
#include <TTimeStepContainerEntry.h>

// ***FORWARDS************************************************************************

class TAggregationResult;
class TSamplingDescription;
class TStatHandler;
class TTimeStep;
class TTopology;

#include "TStatObject.forward.h"

// ***TYPES*************************************************************************** 

typedef TTimeStepContainerEntry < TAggregationResult > TTimedAggregationResult;
typedef TABTree < TTimedAggregationResult > TAggregationResultTable;

// ***CLASSES************************************************************************* 

class TAggregationInfo : public TBaseObject, public TDLNodePrimitive

{
public:

    TAggregationInfo(TParallelID theSamplingID);
    TAggregationInfo(TStatHandler *theResultHandler, 
                     TSamplingDescription *SamplingDescription);
    virtual ~TAggregationInfo(void);

    void
    DisposeResult(TTimeStep &TimeStep);

    virtual TString
    GetInstanceName(void) const;

    TAggregationResult *
    GetResult(TTimeStep &TimeStep, int NrOfCPN = INVALID);

    TStatHandler *
    GetHandler(void);
    
private:

    virtual int
    Compare(TBaseObject *OtherObject, int) const;

    virtual TString
    GetClassName(void) const;

    void
    _Init(void);
    
    TStatHandler            *ResultHandler;
    TParallelID              SamplingID;
    TAggregationResultTable *Results;
};

// ***TYPES*************************************************************************** 

typedef TABTree < TAggregationInfo > TAggregationInfoTable;
typedef TDLList< TAggregationInfo > TAggregationInfoList;
typedef TDLListEnum< TAggregationInfo > TAggregationInfoEnum;

// ***INLINE METHODS******************************************************************
#ifndef NO_INLINE
#endif
// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
