/*      file     : partools/TSamplingDescription.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TSamplingDescription.C,v 1.8 2005/03/29 18:06:46 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <TBaseStream.h>
#include <TDestination.h>
#include <TParallel.h>
#include <TObjectManager.h>

#include "TSamplingDescription.h"

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TSamplingDescription::ClassInfo = 0;

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#  include "TSamplingDescription.i.C"
#endif

// =================================================================================== 
#define ParentClass TParallelListObject
// =================================================================================== 

TSamplingDescription::TSamplingDescription(TParallelID theSamplingID) :

ParentClass(ClassInfo, theSamplingID)

{
    _Init();
}

// =================================================================================== 

TSamplingDescription::TSamplingDescription(TClassID theClassID, int theStatFlags,
                                           TCPNID theDestination, 
                                           TParallelID theSamplingID) :
    
ParentClass(ClassInfo, theSamplingID)

{
    _Init();
    StatObjectClassID = theClassID;
    StatFlags = theStatFlags;
    Destination = theDestination;
}

// =================================================================================== 

bool
TSamplingDescription::Decode(TBaseStream *Stream, TTransferInfo *TransferInfo)

{
    return (ParentClass::Decode(Stream, TransferInfo) AND
            Duration.Decode(Stream) AND
            Frame.Decode(*Stream) AND
            Stream->Decode(StatObjectClassID) AND
            Stream->Decode(Destination) AND     
            Stream->Decode(SampleInterval) AND
            Stream->Decode(CollectInterval) AND
            Stream->Decode(SubTimeStep) AND
            Stream->Decode(StatFlags));
}

// =================================================================================== 

bool
TSamplingDescription::Encode(TBaseStream *Stream, TTransferInfo *TransferInfo) const

{
    return (ParentClass::Encode(Stream, TransferInfo) AND
            Duration.Encode(Stream) AND
            Frame.Encode(*Stream) AND
            Stream->Encode(StatObjectClassID) AND
            Stream->Encode(Destination) AND
            Stream->Encode(SampleInterval) AND
            Stream->Encode(CollectInterval) AND
            Stream->Encode(SubTimeStep) AND
            Stream->Encode(StatFlags));
}

// =================================================================================== 

TString
TSamplingDescription::GetClassName(void) const

{
    return "TSamplingDescription";
}

// =================================================================================== 

TString
TSamplingDescription::GetInstanceName(void) const

{
    char        Line[256];
    static char format[256] = { 0 };
    TClassInfo *ClassInfo = 0;

    if (StatObjectClassID != INVALID)
        ClassInfo = ObjectManager->GetClassInfo(StatObjectClassID);
    if (format[0] == 0)
	sprintf(format, "ID:%s %%s Sample:%%d Collect:%%d", LONG_ID_PRINTF_FORMAT);
    sprintf(Line, format, GetParallelID(),
            ClassInfo? (char *) ClassInfo->GetInstanceName() : "?", 
            SampleInterval, CollectInterval);
    return Line;
}

// =================================================================================== 

void
TSamplingDescription::_Init(void)

{
    DestList = 0;
    Destination = DEST_PARENT;
    StatObjectClassID = INVALID;
    StatFlags = SF_DEFAULT_FLAGS;
    SubTimeStep = 0;
    CollectInterval = 1;
    SampleInterval = INVALID;
}

// =================================================================================== 
#undef ParentClass



