/*      file     : partools/TBoundary.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TBoundary.C,v 1.4 2005/03/25 11:08:14 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <TBaseStream.h>
#include <TObjectManager.h>

#include "TBoundary.h"

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TParallelListObject
// =================================================================================== 

TBoundary::TBoundary(TClassInfo *ClassInfo, TParallelID theParallelID) :

ParentClass(ClassInfo, theParallelID)

{
    Sender = INVALID;
    ObjectClassID = INVALID;
    DistributedObject = 0;
}

// =================================================================================== 

TBoundary::TBoundary(TClassInfo *ClassInfo, TParallelObject *theDistributedObject) :

ParentClass(ClassInfo, theDistributedObject->GetParallelID())

{
    Sender = INVALID;
    DistributedObject = theDistributedObject;
    ObjectClassID = DistributedObject->GetClassID();
}

// =================================================================================== 

bool
TBoundary::Decode(TBaseStream *Stream, TTransferInfo *TransferInfo)

{
    return (ParentClass::Decode(Stream, TransferInfo) AND
            Stream->Decode(ObjectClassID));
}

// =================================================================================== 

bool
TBoundary::Encode(TBaseStream *Stream, TTransferInfo *TransferInfo) const

{
    return (ParentClass::Encode(Stream, TransferInfo) AND
            Stream->Encode(ObjectClassID));
}

// =================================================================================== 

TParallelObject *
TBoundary::GetDistributedObject(void) const

{
    TClassInfo *ClassInfo;

    if (DistributedObject == 0)
    {
        if ((ClassInfo = ObjectManager->GetClassInfo(ObjectClassID)) == 0)
        {
            Error("Decode: cannot retrieve class info of distributed object");
            return 0;
        }
        if ((MutableAccess(TBoundary, DistributedObject) = 
             ClassInfo->GetObjectPtr(GetParallelID(), TRUE)) == 0)
        {
            Error("Decode: could not find distributed object");
            return 0;
        }
    }
    return DistributedObject;
}

// =================================================================================== 

TString
TBoundary::GetClassName(void) const

{
    return "TBoundary";
}

// =================================================================================== 

TString
TBoundary::GetInstanceName(void) const

{
    char Line[256];

    sprintf(Line, "Sender:%lX Class:%d PID:%ld", Sender, ObjectClassID, GetParallelID());
    return Line;
}

// =================================================================================== 

void
TBoundary::SetSender(TCPNID theSender)

{
    Sender = theSender;
}

// =================================================================================== 
#undef ParentClass
